----------------------------------------------------------------------------------------------------
-- (c)2011 Altera Corporation. All rights reserved.
--
-- Altera products are protected under numerous U.S. and foreign patents,
-- maskwork rights, copyrights and other intellectual property laws.
--
-- This reference design file, and your use thereof, is subject to and governed
-- by the terms and conditions of the applicable Altera Reference Design License
-- Agreement (either as signed by you or found at www.altera.com).  By using
-- this reference design file, you indicate your acceptance of such terms and
-- conditions between you and Altera Corporation.  In the event that you do not
-- agree with such terms and conditions, you may not use the reference design
-- file and please promptly destroy any copies you have made.
--
-- This reference design file is being provided on an "as-is�" basis and as an
-- accommodation and therefore all warranties, representations or guarantees of
-- any kind (whether express, implied or statutory) including, without
-- limitation, warranties of merchantability, non-infringement, or fitness for
-- a particular purpose, are specifically disclaimed.  By making this reference
-- design file available, Altera expressly does not recommend, suggest or
-- require that this reference design file be used in combination with any
-- other product not provided by Altera.
----------------------------------------------------------------------------------------------------
-- Author		 : Peter Schepers (pscheper)
----------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity Encoder_64b66b is
PORT(
	Clk  			: in std_logic;
	Enable			: in std_logic;
	Reset	 		: in std_logic;
	kin				: in std_logic_vector(7 downto 0);
	din				: in std_logic_vector(63 downto 0);
	dout	 		: out std_logic_vector(65 downto 0);	
	dout_valid 		: out std_logic;
	encoder_error	: out std_logic
	);
END Encoder_64b66b;	


-- upper 2 bits are for framing 
-- bit[65:64] : "01" => data
--				"10" => control
--				"00" => not valid (used to detect framing)

architecture rtl of Encoder_64b66b is

component scrambler is
GENERIC
(
	WIDTH  : integer range 0 to 512 :=64
);
PORT
(
	clk						: in std_logic; 						
	arst					: in std_logic;	
	ena						: in std_logic;   						
	din						: in std_logic_vector(WIDTH-1 downto 0);	
	dout					: out std_logic_vector(WIDTH-1 downto 0)
 );
 end component;


signal tx_data 				: std_logic_vector(65 downto 0);
signal CountPattern 		: std_logic_vector(63 downto 0);
signal Enable_scrambler		: std_logic;


begin
	



	process(Clk,Reset)
	begin
		if (Reset = '1') then
				tx_data <=  "10" & X"146F146F146FBC1C";
				dout(65 downto 64)	  <=  "10";
				CountPattern 		<= (OTHERS => '0');
				Enable_Scrambler 	<= '0';	
				encoder_error		<= '0';
				dout_valid			<= '0';
		elsif rising_edge(Clk) then	
				
				if Enable = '1' then
					if kin = X"00" then 
						CountPattern <= CountPattern + 1;
						tx_data <= "01" & din;		-- Data
						--tx_data <= "01" & CountPattern;
						--tx_data <=  "01" & X"0000000000000000";
					elsif kin = X"FF" then 
						tx_data <= "10" & din;	--Control
					else
						encoder_error <= '1';
					end if;
					
					dout_valid <= '1';
				else									
					dout_valid <= '0';
				end if;
				
				dout(65 downto 64) <= tx_data(65 downto 64);	-- Framing bits are not scrambled
				Enable_scrambler <= Enable;

		end if;
	end process;



-----------------------------------------------------------------------------------------------------------
--	Scramble the data (note that bits 64 and 65 are not scrambled.
-----------------------------------------------------------------------------------------------------------	

scrambler_inst : scrambler
GENERIC MAP(
	WIDTH	=> 64
)
PORT MAP
(
	clk			=> Clk,						
	arst		=> Reset, 
	ena			=> Enable_Scrambler, 						
	din			=> tx_data(63 downto 0),	
	dout		=> dout(63 downto 0)
);		

			
end;



