library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity Find_Word_Alignment is
PORT(
	RxClock 			: in std_logic;
	Reset	 			: in std_logic;
	rx_syncstatus		: in std_logic_vector(3 downto 0);
	rx_patterndetect	: in std_logic_vector(3 downto 0);
	rx_errdetect		: in std_logic_vector(3 downto 0);
	rx_enapatternalign	: out std_logic;
	Aligned 			: out std_logic;
	SecondAlignment_Found	: out std_logic
	);
END Find_Word_Alignment;	


architecture rtl of Find_Word_Alignment is

type   Aligntype	 		is (Reset_Aligner, Enable_Wordaligner,Alignment_Found,Align_Completed);
signal Alignstate : Aligntype;

signal rx_enapatternalign_i, Aligned_i	: std_logic;
signal rx_patterndetect_combined		: std_logic;
signal Align_Full						: std_logic;
signal rx_syncstatus_combined			: std_logic;
signal rx_errdetect_combined			: std_logic; -- Not used
signal rx_patterndetect_combined_q : std_logic;
signal counter4bit						: std_logic_vector(3 downto 0);


begin


-----------------------------------------------------------------------------------------------------------
--	Align Statement control
-----------------------------------------------------------------------------------------------------------	

	process (RxClock,Reset)
	begin
		if Reset = '1' then
				Alignstate <= Reset_Aligner;
		elsif RxClock'event and RxClock = '1' then
		         CASE Alignstate IS
		            WHEN Reset_Aligner    		=> 	Alignstate <= Enable_Wordaligner;
		            
						WHEN Enable_Wordaligner =>  if rx_syncstatus_combined = '1'  then    -- rx_syncstatus valid on all bytes
																	Alignstate <=  Alignment_Found;
															 end if;
						WHEN Alignment_Found	 	=>  if rx_patterndetect = X"2"   then 			-- Byte correctly re-ordered, Pattern BC in lower half
																	Alignstate <= Align_Completed;																	
															 end if;
						WHEN Align_Completed		=>  if rx_syncstatus_combined = '0' then  	-- Detect something has changed in terms of alignment
																	Alignstate <= Enable_Wordaligner;
																end if;									
								
						When OTHERS					=>  Alignstate <= Reset_Aligner;
				end case;							
		end if;
	end process;
		
	process(RxClock,Reset)
	begin
		if (Reset = '1') then
			rx_enapatternalign_i 		<= '0';
			rx_patterndetect_combined	<= '0';
			rx_errdetect_combined		<= '0';
			rx_syncstatus_combined		<= '0';
			Align_Full 					<= '0';
			SecondAlignment_Found	<= '0';
			rx_patterndetect_combined_q <= '0';
			counter4bit				<= "0000";
		elsif rising_edge(RxClock) then
				if  ( rx_syncstatus = "1111" ) then
					rx_syncstatus_combined 		<= '1';
				else
					rx_syncstatus_combined 		<= '0';
				end if;
				
				rx_errdetect_combined		<= rx_errdetect(0) or rx_errdetect(1) or rx_errdetect(2) or rx_errdetect(3) ;	
				rx_patterndetect_combined 	<= rx_patterndetect(0) or rx_patterndetect(1) or rx_patterndetect(2) or rx_patterndetect(3);
				

		         CASE Alignstate IS
		            WHEN Reset_Aligner    		=> 	rx_enapatternalign_i <= '0';
																	Align_Full <= '0';
		            
						WHEN Enable_Wordaligner =>  rx_enapatternalign_i <= '1';				
--															 counter4bit <= counter4bit + 1;
--															 if counter4bit = "000" then
--																	rx_enapatternalign_i <= '1';
--															 else
--																	rx_enapatternalign_i <= '0';
--															
--															 end if;
															 	--rx_enapatternalign_i <= not rx_enapatternalign_i;
															 Align_Full <= '0';
						
						WHEN Alignment_Found	 	=>  rx_enapatternalign_i <= '0';
															 Align_Full <= '0';
						WHEN Align_Completed		=>  Align_Full <= '1';								
																assert(Align_Full = '1')
																report "==========================================================================================================> Lane Aligned ..." 		severity note;						
								
						When OTHERS					=>  rx_enapatternalign_i <= '0';
															 Align_Full <= '0';
				end case;					
				
				rx_patterndetect_combined_q <= rx_patterndetect_combined;
				
				if (Align_Full = '1') then
					if (rx_patterndetect_combined_q /= rx_patterndetect_combined) and (rx_patterndetect_combined = '1' ) then
						SecondAlignment_Found <= '1';
					end if;
				else
						SecondAlignment_Found <= '0';
				end if;
					
		end if;
	end process;
		

Aligned <= Align_Full;
rx_enapatternalign <= rx_enapatternalign_i;
			
end;



