-- A10GX SI Board (10AX115S2F45I1SG)
-- 4 Channel Superlite II V3 Demo Design V16.1.2  (QII 16.1.2 B203)
-- 4 Bidirectional Channels (Tx+Rx) at 10.3125 Gbps  (64b66b encoded), mapped to QSFP+ Module
-- Contains handshaking between Rx and Tx as well as Flow Control (XON/XOFF).
-- Uses RxFifo inside the transceivers to do the deskew
-- Using Native Phy Basic Enhanced PCS Mode
--
-- Last Update : April 14th 2017
-- Author : Peter Schepers (peter.schepers@intel.com)

LIBRARY ieee;
USE ieee.std_logic_1164.all; 

LIBRARY work;

ENTITY devkit_demo IS 
	PORT
	(
	
		LVDS_CLK3p 				:  IN  STD_LOGIC; -- 100 Mhz input clock for mgmt_clk and reconfiguration
		
		-- QSFP module (4 lanes)

		REFCLKGXBL_1E_Tp		:	 IN STD_LOGIC; -- 644.53125 Mhz per default (Y3)
		
		GXBL_1E_RX0p    :     IN STD_LOGIC;
		GXBL_1E_RX1p    :     IN STD_LOGIC;
		GXBL_1E_RX3p    :     IN STD_LOGIC;
		GXBL_1E_RX4p    :     IN STD_LOGIC;

		
		GXBL_1E_TX0p    :     OUT STD_LOGIC;
		GXBL_1E_TX1p    :     OUT STD_LOGIC;
		GXBL_1E_TX3p    :     OUT STD_LOGIC;
		GXBL_1E_TX4p    :     OUT STD_LOGIC;

		-- QSFP Status interface
		eQSFP_modselL	: 		 out std_logic;
		eQSFP_resetL	:		 out std_logic;
		eQSFP_LPmode	:		 out std_logic;
		eQSFP_modprsL	:		 in std_logic;
		eQSFP_intL		:      in std_logic;
		
		
		USER_LED0 :  OUT  STD_LOGIC; -- TxPLL Locked (transceiver)
		USER_LED1 :  OUT  STD_LOGIC; -- rx_freqlocked
		USER_LED2 :  OUT  STD_LOGIC; -- WordAligned
		USER_LED3 :  OUT  STD_LOGIC; -- LaneAligned
		USER_LED4 :  OUT  STD_LOGIC; -- LinkUp
		USER_LED5 :  OUT  STD_LOGIC; -- Locked (CountLocked and all PRBSLocked signal asserted)
		USER_LED6 :  OUT  STD_LOGIC; -- Link OK :  Locked + No biterrors
		USER_LED7 :  OUT  STD_LOGIC  -- XOFF Received
	);
END devkit_demo;




ARCHITECTURE rtl OF devkit_demo IS 



component SuperliteII_Demo is
PORT(
	RefClock						: in std_logic; 						
	Clk100Mhz					: in std_logic;
	Enable_Core					: in std_logic;
	XCVR_TX						: OUT STD_LOGIC_VECTOR (3 DOWNTO 0);	-- 4 Channels
	XCVR_RX						: IN STD_LOGIC_VECTOR (3 DOWNTO 0);     
	led							: out std_logic_vector(7 downto 0);
	Control_Reg					: in std_logic_vector(15 downto 0);
	Control2_Reg				: in std_logic_vector(15 downto 0);
	reconfig_reset          : in  std_logic; 
	reconfig_write          : in  std_logic; 
	reconfig_read           : in  std_logic; 
	reconfig_address        : in  std_logic_vector(15 downto 0)  := (others => '0'); 
	reconfig_writedata      : in  std_logic_vector(31 downto 0)  := (others => '0'); 
	reconfig_readdata       : out std_logic_vector(31 downto 0);   
	reconfig_waitrequest    : out  std_logic;
	reconfig_pll_reset          : in  std_logic; 
	reconfig_pll_write          : in  std_logic; 
	reconfig_pll_read           : in  std_logic; 
	reconfig_pll_address        : in  std_logic_vector(15 downto 0)  := (others => '0'); 
	reconfig_pll_writedata      : in  std_logic_vector(31 downto 0)  := (others => '0'); 
	reconfig_pll_readdata       : out std_logic_vector(31 downto 0);   
	reconfig_pll_waitrequest    : out  std_logic;
	Channel_Reg					: out std_logic_vector(15 DOWNTO 0);
	Counter_1ms_Reg			: out std_logic_vector(31 DOWNTO 0);
	ErrorCount_Reg				: out std_logic_vector(15 DOWNTO 0);
	Bitrate_Reg					: out std_logic_vector(31 DOWNTO 0);
	DataClock_Reg				: out std_logic_vector(31 DOWNTO 0);
	DataOut_Clock_Reg			: out std_logic_vector(31 DOWNTO 0);
	Latency_Max_Reg			: out std_logic_vector(7 downto 0);
	Latency_Min_Reg			: out std_logic_vector(7 downto 0);	
	Delay_Reg					: out std_logic_vector(31 downto 0) := (OTHERS => '0');
	Delay_Reg_2					: out std_logic_vector(31 downto 0) := (OTHERS => '0');
	--FEC_correctable_error_Reg				: out std_logic_vector(31 DOWNTO 0):= (OTHERS => '0');
	--FEC_uncorrectable_error_Reg			: out std_logic_vector(15 DOWNTO 0):= (OTHERS => '0');
	Temperature					: out std_logic_vector(7 downto 0)	
	);
END component SuperliteII_Demo;
	
	component controller is
		port (
			in_port_to_the_BitErrorCount_Reg    : in  std_logic_vector(15 downto 0) := (others => 'X'); -- export
			in_port_to_the_Bitrate_Reg          : in  std_logic_vector(31 downto 0) := (others => 'X'); -- export
			in_port_to_the_Channel_Reg          : in  std_logic_vector(15 downto 0) := (others => 'X'); -- export
			clk                                 : in  std_logic                     := 'X';             -- clk
			reset_n                             : in  std_logic                     := 'X';             -- reset_n
			out_port_from_the_Control2_Reg      : out std_logic_vector(15 downto 0);                    -- export
			out_port_from_the_Control_Reg       : out std_logic_vector(15 downto 0);                    -- export
			count_reset_reg_export              : in  std_logic_vector(31 downto 0) := (others => 'X'); -- export
			in_port_to_the_Counter_1ms_Reg      : in  std_logic_vector(31 downto 0) := (others => 'X'); -- export
			dataclock_reg_export                : in  std_logic_vector(31 downto 0) := (others => 'X'); -- export
			dataout_clock_reg_export            : in  std_logic_vector(31 downto 0) := (others => 'X'); -- export
			delay_reg_export                    : in  std_logic_vector(31 downto 0) := (others => 'X'); -- export
			delay_reg_2_export                  : in  std_logic_vector(31 downto 0) := (others => 'X'); -- export
			fec_correctable_error_reg_export    : in  std_logic_vector(31 downto 0) := (others => 'X'); -- export
			fec_uncorrectable_error_reg_export  : in  std_logic_vector(15 downto 0) := (others => 'X'); -- export
			latency_max_reg_export              : in  std_logic_vector(7 downto 0)  := (others => 'X'); -- export
			latency_min_reg_export              : in  std_logic_vector(7 downto 0)  := (others => 'X'); -- export
			s0_address_from_the_reconfig_mgmt   : out std_logic_vector(15 downto 0);                    -- address
			s0_read_from_the_reconfig_mgmt      : out std_logic;                                        -- read
			s0_readdata_to_the_reconfig_mgmt    : in  std_logic_vector(31 downto 0) := (others => 'X'); -- readdata
			s0_write_from_the_reconfig_mgmt     : out std_logic;                                        -- write
			s0_writedata_from_the_reconfig_mgmt : out std_logic_vector(31 downto 0);                    -- writedata
			s0_waitrequest_to_the_reconfig_mgmt : in  std_logic                     := 'X';             -- waitrequest
			reconfig_mgmt_pll_reset_reset       : out std_logic;                                        -- reset
			reconfig_mgmt_pll_s0_address        : out std_logic_vector(15 downto 0);                    -- address
			reconfig_mgmt_pll_s0_read           : out std_logic;                                        -- read
			reconfig_mgmt_pll_s0_readdata       : in  std_logic_vector(31 downto 0) := (others => 'X'); -- readdata
			reconfig_mgmt_pll_s0_write          : out std_logic;                                        -- write
			reconfig_mgmt_pll_s0_writedata      : out std_logic_vector(31 downto 0);                    -- writedata
			reconfig_mgmt_pll_s0_waitrequest    : in  std_logic                     := 'X';             -- waitrequest
			reset_reset_from_the_reconfig_mgmt  : out std_logic;                                        -- reset
			temperature_reg_export              : in  std_logic_vector(7 downto 0)  := (others => 'X')  -- export
		);
	end component controller;



SIGNAL	BitErrorCount_Reg :  STD_LOGIC_VECTOR(15 DOWNTO 0);
SIGNAL	Bitrate_Reg :  STD_LOGIC_VECTOR(31 DOWNTO 0);

SIGNAL	Channel_Reg :  STD_LOGIC_VECTOR(15 DOWNTO 0);
SIGNAL	Clk100Mhz :  STD_LOGIC;
SIGNAL	Control2_Reg_CPU :  STD_LOGIC_VECTOR(15 DOWNTO 0);
SIGNAL	Control_Reg_CPU :  STD_LOGIC_VECTOR(15 DOWNTO 0);
SIGNAL	Counter_1ms_Reg :  STD_LOGIC_VECTOR(31 DOWNTO 0);
SIGNAL	DataClock_Reg :  STD_LOGIC_VECTOR(31 DOWNTO 0);
SIGNAL   DataOut_Clock_Reg :  STD_LOGIC_VECTOR(31 DOWNTO 0);
SIGNAL	Latency_Max_Reg :  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL	Latency_Min_Reg :  STD_LOGIC_VECTOR(7 DOWNTO 0);

SIGNAL 	Delay_Reg :  STD_LOGIC_VECTOR(31 DOWNTO 0);
SIGNAL 	Delay_Reg_2 :  STD_LOGIC_VECTOR(31 DOWNTO 0);
SIGNAL 	FEC_correctable_error_Reg :  STD_LOGIC_VECTOR(31 DOWNTO 0);
SIGNAL 	FEC_uncorrectable_error_Reg :  STD_LOGIC_VECTOR(15 DOWNTO 0);
			
SIGNAL	led :  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL	reconfig_address :  STD_LOGIC_VECTOR(15 DOWNTO 0);
SIGNAL	reconfig_pll_address :  STD_LOGIC_VECTOR(15 DOWNTO 0);
SIGNAL	reconfig_pll_read :  STD_LOGIC;
SIGNAL	reconfig_pll_readdata :  STD_LOGIC_VECTOR(31 DOWNTO 0);
SIGNAL	reconfig_pll_reset :  STD_LOGIC;
SIGNAL	reconfig_pll_waitrequest :  STD_LOGIC;
SIGNAL	reconfig_pll_write :  STD_LOGIC;
SIGNAL	reconfig_pll_writedata :  STD_LOGIC_VECTOR(31 DOWNTO 0);
SIGNAL	reconfig_read :  STD_LOGIC;
SIGNAL	reconfig_readdata :  STD_LOGIC_VECTOR(31 DOWNTO 0);
SIGNAL	reconfig_reset :  STD_LOGIC;
SIGNAL	reconfig_waitrequest :  STD_LOGIC;
SIGNAL	reconfig_write :  STD_LOGIC;
SIGNAL	reconfig_writedata :  STD_LOGIC_VECTOR(31 DOWNTO 0);
SIGNAL	RefClock :  STD_LOGIC;
SIGNAL	RX :  STD_LOGIC_VECTOR(3 DOWNTO 0);
SIGNAL	Temp_Sense :  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL	TX :  STD_LOGIC_VECTOR(3 DOWNTO 0);


BEGIN 


-- Map signalnames to Arria10 Signal Integrity board names.

RefClock 	<= REFCLKGXBL_1E_Tp;
Clk100Mhz 	<= LVDS_CLK3p;


RX(0) <= GXBL_1E_RX0p;
RX(1) <= GXBL_1E_RX1p;
RX(2) <= GXBL_1E_RX3p;
RX(3) <= GXBL_1E_RX4p;


GXBL_1E_TX0p <= TX(0);
GXBL_1E_TX1p <= TX(1);
GXBL_1E_TX3p <= TX(2);
GXBL_1E_TX4p <= TX(3);

USER_LED0 <= led(0);
USER_LED1 <= led(1);
USER_LED2 <= led(2);
USER_LED3 <= led(3);
USER_LED4 <= led(4);
USER_LED5 <= led(5);
USER_LED6 <= led(6);
USER_LED7 <= led(7);

----------------------------------------------------------------------------------------
-- Instantiate Superlite_Demo
----------------------------------------------------------------------------------------


inst2 : superliteii_demo
PORT MAP(

		 RefClock 				=> RefClock,
		 Clk100Mhz 				=> Clk100Mhz,
		 Enable_Core 			=> '1',

		 XCVR_RX 				=> RX,
		 XCVR_TX 				=> TX,
		 
		 Control_Reg 			=> Control_Reg_CPU,
		 Control2_Reg 			=> Control2_Reg_CPU,
		 
		 reconfig_reset 			=> reconfig_reset,
		 reconfig_write 			=> reconfig_write,
		 reconfig_writedata 		=> reconfig_writedata,		 
		 reconfig_read 			=> reconfig_read,
		 reconfig_readdata 		=> reconfig_readdata,		 
		 reconfig_address 		=> reconfig_address,
		 reconfig_waitrequest 	=> reconfig_waitrequest,
		 
		 reconfig_pll_reset 		=> reconfig_pll_reset,
		 reconfig_pll_write 		=> reconfig_pll_write,
		 reconfig_pll_writedata => reconfig_pll_writedata,		 
		 reconfig_pll_read 		=> reconfig_pll_read,	
		 reconfig_pll_readdata 	=> reconfig_pll_readdata,		 
		 reconfig_pll_address 	=> reconfig_pll_address,
		 reconfig_pll_waitrequest => reconfig_pll_waitrequest,

		 Bitrate_Reg 			=> Bitrate_Reg,
		 Channel_Reg 			=> Channel_Reg,
		 Counter_1ms_Reg 		=> Counter_1ms_Reg,
		 DataClock_Reg 		=> DataClock_Reg,
		 DataOut_Clock_Reg	=> DataOut_Clock_Reg,
		 ErrorCount_Reg 		=> BitErrorCount_Reg,
		 Latency_Max_Reg 		=> Latency_Max_Reg,	 
		 Latency_Min_Reg 		=> Latency_Min_Reg,
		 Delay_Reg				=> Delay_Reg,
		 Delay_Reg_2			=> Delay_Reg_2,
		 led 						=> led,
		 Temperature 			=> Temp_Sense 
		 );

----------------------------------------------------------------------------------------
-- Instantiate Qsys system with Nios II
----------------------------------------------------------------------------------------


controller_inst : component controller
		port map (
			in_port_to_the_BitErrorCount_Reg    => BitErrorCount_Reg,    --      biterrorcount_reg.export
			in_port_to_the_Bitrate_Reg          => Bitrate_Reg,          --            bitrate_reg.export
			in_port_to_the_Channel_Reg          => Channel_Reg,          --            channel_reg.export
			clk                                 => Clk100Mhz,                                --             clk_clk_in.clk
			reset_n                             => '1',                             --       clk_clk_in_reset.reset_n
			out_port_from_the_Control2_Reg      => Control2_Reg_CPU,      --           control2_reg.export
			out_port_from_the_Control_Reg       => Control_Reg_CPU,       --            control_reg.export
			in_port_to_the_Counter_1ms_Reg      => Counter_1ms_Reg,      --        counter_1ms_reg.export
			dataclock_reg_export                => DataClock_Reg,                --          dataclock_reg.export
			dataout_clock_reg_export            => DataOut_Clock_Reg,            --      dataout_clock_reg.export
			delay_reg_export                    => Delay_Reg,
			delay_reg_2_export                  => Delay_Reg_2,
			fec_correctable_error_reg_export    => (OTHERS => '0'), -- not used
			fec_uncorrectable_error_reg_export  => (OTHERS => '0'), -- not used
			latency_max_reg_export              => Latency_Max_Reg,
			latency_min_reg_export              => Latency_Min_Reg,			
			reset_reset_from_the_reconfig_mgmt  => reconfig_reset,  --    reconfig_mgmt_reset.reset			
			s0_address_from_the_reconfig_mgmt   => reconfig_address,   --        reconfig_mgmt_0.address
			s0_read_from_the_reconfig_mgmt      => reconfig_read,      --                       .read
			s0_readdata_to_the_reconfig_mgmt    => reconfig_readdata,    --                       .readdata
			s0_write_from_the_reconfig_mgmt     => reconfig_write,     --                       .write
			s0_writedata_from_the_reconfig_mgmt => reconfig_writedata, --                       .writedata
			s0_waitrequest_to_the_reconfig_mgmt => reconfig_waitrequest, --                       .waitrequest
			reconfig_mgmt_pll_reset_reset        => reconfig_pll_reset,        -- reconfig_mgmt_pll_reset.reset
			reconfig_mgmt_pll_s0_address         => reconfig_pll_address,         --    reconfig_mgmt_pll_s0.address
			reconfig_mgmt_pll_s0_read            => reconfig_pll_read,            --                       .read
			reconfig_mgmt_pll_s0_readdata        => reconfig_pll_readdata,        --                       .readdata
			reconfig_mgmt_pll_s0_write           => reconfig_pll_write,           --                       .write
			reconfig_mgmt_pll_s0_writedata       => reconfig_pll_writedata,       --                       .writedata
			reconfig_mgmt_pll_s0_waitrequest     => reconfig_pll_waitrequest,     --                       .waitrequest
			temperature_reg_export              => Temp_Sense               --        temperature_reg.export
		);

-----------------------------------------------------------------------------------------------------------
--	QSFP+ control signals
-----------------------------------------------------------------------------------------------------------			  
	
eQSFP_modselL	<= '1';
eQSFP_resetL	<= '1';
eQSFP_LPmode	<= '0';

	

END rtl;