# ttk_helper.tcl
# Tcl procedures for TTK designs

puts {===============================================================================================}
puts {This tcl file contains a number of procedures to do additional configuration and status updates}
puts {Revision 4.0}
puts {Date : 21 October 2015}
puts {Author : Peter Schepers}
puts {===============================================================================================}
puts ""



###############################################################################
# show_phys : list the native phy's in the design
# Once identified they can be addressed using the index
###############################################################################

proc show_phys {} {
set phy_list {}
set i 0 
foreach m [get_service_paths slave] {
     
     if {[string last slave $m] != -1 } {
           puts ""
          # Determine from text in the marker info if m is a Native PHY and only continue if it is
          # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
           set marker_info [marker_get_info $m] 

           set m [claim_service slave $m ""]
           if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {
                     set slave_addr [dict get $marker_info BASE_ADDRESS]
                     set identifier [rd $m 0x0 0x200]
                     puts -nonewline "Phy $i  :"
                     puts -nonewline "Slave: [format 0x%x $slave_addr]  "
                     puts -nonewline "Identifier: $identifier  "
                     puts ""
                     puts "Marker info (unique identifier): $marker_info  "
           incr i
           lappend phy_list $marker_info
           }
}
}
close_service slave $m
return $phy_list
}



	


###############################################################################
# read register
###############################################################################
proc rd {m base_addr offset } {
  set value [master_read_8 $m [expr $base_addr + $offset * 4] 1]

}
#puts {rd m base_addr offset }
#puts {example usage: rd $m $base_addr 0x138}
#puts ""

proc rd_mask_dec {m base_addr offset bitmask } {
  set value [master_read_8 $m [expr $base_addr + $offset * 4] 1]
  puts -nonewline [format %d [expr $value & $bitmask]]
}

proc rd_mask_hex {m base_addr offset bitmask } {
  set value [master_read_8 $m [expr $base_addr + $offset * 4] 1]
  puts -nonewline [format 0x%x [expr $value & $bitmask]]
}


###############################################################################
# read modify write register
###############################################################################

proc rmw {m base_addr offset bitmask newval} {
  # mask out the newval so that un-masked bits do not change
  set newval_masked [expr $newval & $bitmask]

  # read from data register
  set value [master_read_8 $m [expr $base_addr + $offset * 4] 1]

  # bitwise-AND with the negative of bitmask to clear out only the bitmask bits
  set value [expr $value & [expr 0xff & ~$bitmask]] 

  # bitwise-OR this data with the newval_masked so that only the masked bits get changed
  set value [expr $value | $newval_masked] 

  # write the new data back
  master_write_8 $m [expr $base_addr + $offset * 4] $value
}
#puts {rmw {m base_addr offset bitmask newval} }
#puts {example usage: rmw $m $base_addr 0x160 0xE 0x4}
#puts ""

	
###############################################################################		
#Do rmw for all channels
###############################################################################
proc rmw_all {offset mask newval} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
          # Determine from text in the marker info if m is a Native PHY and only continue if it is
          # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
           set marker_info [marker_get_info $m] 

           set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {
			
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]
		
				rmw $m $base_addr $offset $mask $newval

				
				
				# Print info
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				set identifier [rd $m $base_addr 0x200]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x $base_addr]	"
				puts -nonewline "	Offset : [format 0x%x $offset] " 
				set read_back [rd $m $base_addr $offset]
				set read_back_mask [expr [expr $read_back & $mask ]]
				puts -nonewline "	Newval : [format 0x%x $read_back_mask] " 
				puts ""
}}}}
close_service slave $m
}

puts {rmw_all {offset mask newval}}
puts {example useage rmw_all 0x160 0x0E 0x03}
puts ""

###############################################################################		
#Do rmw for specific channel within a phy
###############################################################################
proc rmw_channel {phy index channel offset mask newval} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
         # Determine from text in the marker info if m is a Native PHY and only continue if it is
          # The marker will exist on the path of $m right now, but we're going to change the path in a second so do this now.
           set marker_info [marker_get_info $m] 

           set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {
			
		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 

				set base_addr [expr 0x1000 * $channel]
		
				rmw $m $base_addr $offset $mask $newval

				
				
				# Print info
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "channel  : $channel	"
				puts -nonewline "base_addr : [format 0x%x $base_addr]	"
				puts -nonewline "	Offset : [format 0x%x $offset] " 
				set read_back [rd $m $base_addr $offset]
				set read_back_mask [expr [expr $read_back & $mask ]]
				puts -nonewline "	Newval : [format 0x%x $read_back_mask] " 
				puts ""

}}}}
close_service slave $m
}

puts {rmw_channel {phy index channel offset mask newval}}
puts {example useage rmw_channel $phy 1 3 0x134 0x0F 0x03}
puts ""

###############################################################################		
#read a register from a specific channel in a phy
###############################################################################
proc rd_channel {phy index channel offset} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
			
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {
			
		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 

				set base_addr [expr 0x1000 * $channel]

				
				
				# Print info
				set identifier [rd $m 0x0 0x200]
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $channel	"
				puts -nonewline "base_addr : [format 0x%x $base_addr]	"
				puts -nonewline "Offset : [format 0x%x $offset] " 
				set read_back [rd $m $base_addr $offset]
				puts -nonewline "	Read : [format 0x%x $read_back] " 
				puts ""
}}}}
close_service slave $m
}
puts {rd_channel {phy index channel offset}}
puts {example useage rd_channel $phy 0 3 0x134}
puts ""

###############################################################################		
#Do read of a register for all channels
###############################################################################
proc rd_all {offset} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
          # Determine from text in the marker info if m is a Native PHY and only continue if it is
          # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
           set marker_info [marker_get_info $m] 

           set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {
			
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]
		


				
				
				# Print info
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				set identifier [rd $m $base_addr 0x200]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x $base_addr]	"
				puts -nonewline "	Offset : [format 0x%x $offset] " 
				set read_back [rd $m $base_addr $offset ]
				puts -nonewline "	ReadValue : [format 0x%x $read_back] " 
				puts ""
}}}}
close_service slave $m
}

puts {rd_all {offset}}
puts {example useage rd_all 0x160}
puts ""


###############################################################################		
#Set VGA Gain to $vga_gain for all channels
###############################################################################
proc set_vga_gain_all {vga_gain} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {
			
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]

				# Set VGA Gain 0x160 bit [3:1]
				# rmw_channel( ((j<<10) + 0x160),0xE, vga_gain[j] << 1);				
				rmw $m $base_addr 0x160 0xE [expr $vga_gain << 1]

				
				
				# Print info
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				set identifier [rd $m $base_addr 0x200]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x $base_addr]	"
				puts -nonewline "	VGA Gain :  " 
				#	 vga_gain[SelectedChannel]      = (rd_channel((SelectedChannel << 10) + 0x160) & 0xE) >> 1;
				set read_back [rd $m $base_addr 0x160]
				puts -nonewline [expr [expr $read_back & 0xE ] >> 1]
				puts ""
}}}}
close_service slave $m
}

puts {set_vga_gain_all {vga_gain}}
puts {example useage set_vga_gain_all 5}
puts ""


###############################################################################		
#Get VGA Gain from all channels
###############################################################################
proc get_vga_gain_all {} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
         # Determine from text in the marker info if m is a Native PHY and only continue if it is
          # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
           set marker_info [marker_get_info $m] 

           set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {
		
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]


				# Print info
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				set identifier [rd $m $base_addr 0x200]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x $base_addr]	"
				puts -nonewline "	VGA Gain :  " 
				#	 vga_gain[SelectedChannel]      = (rd_channel((SelectedChannel << 10) + 0x160) & 0xE) >> 1;
				set read_back [rd $m $base_addr 0x160]
				puts -nonewline [expr [expr $read_back & 0xE ] >> 1]
				puts ""
}}}}
close_service slave $m
}

puts {get_vga_gain_all {}}
puts {example useage set_vga_gain_all }
puts ""

###############################################################################
#Set VGA Gain to $vga_gain to $phy
###############################################################################
proc set_vga_gain_phy {phy index vga_gain} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 
		
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]

				# Set VGA Gain 0x160 bit [3:1]
				# rmw_channel( ((j<<10) + 0x160),0xE, vga_gain[j] << 1);				
				rmw $m $base_addr 0x160 0xE [expr $vga_gain << 1]


				# Print info
	
				set identifier [rd $m $base_addr 0x200]
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x [expr $base_addr]]	"
				puts -nonewline "	VGA Gain :  " 
				#	 vga_gain[SelectedChannel]      = (rd_channel((SelectedChannel << 10) + 0x160) & 0xE) >> 1;
				set read_back [rd $m $base_addr 0x160]
				puts -nonewline [expr [expr $read_back & 0xE ] >> 1]
				puts ""
}}}}}
close_service slave $m
}

puts {set_vga_gain_phy {phy index vga_gain}}
puts {example useage set_vga_gain_phy $phy 1 5}
puts ""

###############################################################################
#Get VGA Gain from all channels in $phy
###############################################################################
proc get_vga_gain_phy {phy index} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 
		
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]
				 
				set identifier [rd $m $base_addr 0x200]
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x [expr $base_addr]]	"
				puts -nonewline "	VGA Gain :  " 
				#	 vga_gain[SelectedChannel]      = (rd_channel((SelectedChannel << 10) + 0x160) & 0xE) >> 1;
				set read_back [rd $m $base_addr 0x160]
				puts -nonewline [expr [expr $read_back & 0xE ] >> 1]
				puts ""
}}}}}
close_service slave $m
}

puts {get_vga_gain_phy {phy index}}
puts {example useage get_vga_gain_phy $phy 1 }
puts ""


###############################################################################
#Set VGA Gain to $vga_gain to $channel in $phy
###############################################################################
proc set_vga_gain_channel {phy index channel vga_gain} {

foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 

set base_addr [expr [expr 0x1000 * $channel] ]

# Set VGA Gain 0x160 bit [3:1]
# rmw_channel( ((j<<10) + 0x160),0xE, vga_gain[j] << 1);				
rmw $m $base_addr 0x160 0xE [expr $vga_gain << 1]


# Print info

set identifier [rd $m $base_addr 0x200]
set slave_addr [dict get $marker_info BASE_ADDRESS]
puts -nonewline "Slave: [format 0x%x $slave_addr]  "
puts -nonewline "Identifier: $identifier  "
puts -nonewline "channel  : $channel	"
puts -nonewline "base_addr : [format 0x%x [expr $base_addr]]	"

puts -nonewline "	VGA Gain :  " 
#	 vga_gain[SelectedChannel]      = (rd_channel((SelectedChannel << 10) + 0x160) & 0xE) >> 1;
set read_back [rd $m $base_addr 0x160]
puts -nonewline [expr [expr $read_back & 0xE ] >> 1]
puts ""
}}}}
close_service slave $m
}

puts {set_vga_gain_channel {phy index channel vga_gain}}
puts {example useage set_vga_gain_channel $phy 1 2 5}
puts ""


###############################################################################		
#Set AC Gain to $ac_gain for all channels
###############################################################################
proc set_ac_gain_all {ac_gain} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {
			
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]


	 			# Set AC Gain 0x167 bit [5:1]
				# rmw_channel( ((j<<10) + 0x167),0x3E, rx_eqctrl[j] << 1);			
				rmw $m $base_addr 0x167 0x3E [expr $ac_gain << 1]

				
				
				# Print info
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				set identifier [rd $m $base_addr 0x200]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x $base_addr]	"
				puts -nonewline "	AC Gain :  " 
				# rx_eqctrl[SelectedChannel]     = (rd_channel((SelectedChannel << 10) + 0x167) & 0x3E) >> 1;
				set read_back [rd $m $base_addr 0x167]
				puts -nonewline [expr [expr $read_back & 0x3E ] >> 1]
				puts ""
}}}}
close_service slave $m
}

puts {set_ac_gain_all {ac_gain}}
puts {example useage set_ac_gain_all 5}
puts ""


###############################################################################		
#Get AC Gain from all channels
###############################################################################
proc get_ac_gain_all {} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {
		
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]


				# Print info
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				set identifier [rd $m $base_addr 0x200]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x $base_addr]	"
				puts -nonewline "	AC Gain :  " 
				# rx_eqctrl[SelectedChannel]     = (rd_channel((SelectedChannel << 10) + 0x167) & 0x3E) >> 1;
				set read_back [rd $m $base_addr 0x167]
				puts -nonewline [expr [expr $read_back & 0x3E ] >> 1]
				puts ""
}}}}
close_service slave $m
}

puts {get_ac_gain_all {}}
puts {example useage set_ac_gain_all }
puts ""

###############################################################################
#Set AC Gain to $ac_gain to $phy
###############################################################################
proc set_ac_gain_phy {phy index ac_gain} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 
		
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]

	 			# Set AC Gain 0x167 bit [5:1]
				# rmw_channel( ((j<<10) + 0x167),0x3E, rx_eqctrl[j] << 1);			
				rmw $m $base_addr 0x167 0x3E [expr $ac_gain << 1]


				# Print info
	
				set identifier [rd $m $base_addr 0x200]
				set slave_addr [dict get $marker_info BASE_ADDRESS]				
				puts -nonewline "Slave: [format 0x%x $slave_addr]   "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x [expr $base_addr]]	"
				puts -nonewline "	AC Gain :  " 
				# rx_eqctrl[SelectedChannel]     = (rd_channel((SelectedChannel << 10) + 0x167) & 0x3E) >> 1;
				set read_back [rd $m $base_addr 0x167]
				puts -nonewline [expr [expr $read_back & 0x3E ] >> 1]
				puts ""
}}}}}
close_service slave $m
}

puts {set_ac_gain_phy {phy index ac_gain}}
puts {example useage set_ac_gain_phy $phy 1 5}
puts ""

###############################################################################
#Get AC Gain from all channels in $phy
###############################################################################
proc get_ac_gain_phy {phy index} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 
		
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]
				 
				set identifier [rd $m $base_addr 0x200]
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x [expr $base_addr ]]	"
				puts -nonewline "	AC Gain :  " 
				# rx_eqctrl[SelectedChannel]     = (rd_channel((SelectedChannel << 10) + 0x167) & 0x3E) >> 1;
				set read_back [rd $m $base_addr 0x167]
				puts -nonewline [expr [expr $read_back & 0x3E ] >> 1]
				puts ""
}}}}}
close_service slave $m
}

puts {get_ac_gain_phy {phy index}}
puts {example useage get_ac_gain_phy $phy 1 }
puts ""


###############################################################################
#Set AC Gain to $ac_gain to $channel in $phy
###############################################################################
proc set_ac_gain_channel {phy index channel ac_gain} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 

set base_addr [expr [expr 0x1000 * $channel] ]

# Set AC Gain 0x167 bit [5:1]
# rmw_channel( ((j<<10) + 0x167),0x3E, rx_eqctrl[j] << 1);			
rmw $m $base_addr 0x167 0x3E [expr $ac_gain << 1]


# Print info
set identifier [rd $m $base_addr 0x200]
set slave_addr [dict get $marker_info BASE_ADDRESS]
puts -nonewline "Slave: [format 0x%x $slave_addr]  "
puts -nonewline "channel  : $channel	"
puts -nonewline "base_addr : [format 0x%x [expr $base_addr]]	"

puts -nonewline "	AC Gain :  " 
# rx_eqctrl[SelectedChannel]     = (rd_channel((SelectedChannel << 10) + 0x167) & 0x3E) >> 1;
set read_back [rd $m $base_addr 0x167]
puts -nonewline [expr [expr $read_back & 0x3E ] >> 1]
puts ""
}}}}
close_service slave $m
}

puts {set_ac_gain_channel {phy channel index ac_gain}}
puts {example useage set_ac_gain_channel $phy 1 2 5}
puts ""


###############################################################################
#Set CDR gear to $dyn $fix  and $ipd for all channels
###############################################################################
proc set_cdr_gear_all {dyn fix ipd} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {
		
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]


		#Set dyn to 31 : 0x137 bits [6:2]
		#rmw_channel( ((j<<10) + 0x137),0x7C,0x7C);
		
		rmw $m $base_addr 0x137 0x7C [expr $dyn << 2]
		
		#Set fix 
		set fix_3_0 [expr $fix & 0x0F]
		set fix_4   [expr [expr $fix & 0x10] >> 4]
		set fix_5   [expr [expr $fix & 0x20] >> 5]
		set fix_6   [expr [expr $fix & 0x40] >> 6]
		set fix_7   [expr [expr $fix & 0x80] >> 7]
		rmw $m $base_addr 0x136 0x0F $fix_3_0
		rmw $m $base_addr 0x134 0x40 [expr $fix_4 << 6]
		rmw $m $base_addr 0x135 0x40 [expr $fix_5 << 6]
		rmw $m $base_addr 0x132 0x04 [expr $fix_6 << 2]
		rmw $m $base_addr 0x132 0x01 [expr $fix_7 << 0]
		
		#set IPD 0x139 bit [5:3]
		#rmw_channel( ((j<<10) + 0x139),0x38,0x18);	
		rmw $m $base_addr 0x139 0x38 [expr $ipd << 3]

				# Print info
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				set identifier [rd $m $base_addr 0x200]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x $base_addr]	"
				puts -nonewline "	Dyn  : " 
				set read_back [rd $m $base_addr 0x137]
				puts -nonewline  [expr [expr $read_back & 0x7C] >> 2]
				set read_back [rd $m $base_addr 0x136]
				set fix_3_0 [expr $read_back & 0x0F]
				set read_back [rd $m $base_addr 0x134]
				set temp [expr [expr $read_back & 0x40] >> 6]
				set fix_4 [expr $temp << 4]
				set read_back [rd $m $base_addr 0x135]
				set temp	[expr [expr $read_back & 0x40] >> 6]	
				set fix_5 [expr $temp << 5]
				set read_back [rd $m $base_addr 0x132]
				set temp [expr [expr $read_back & 0x04] >> 2]	
				set fix_6 [expr $temp << 6]
				set read_back [rd $m $base_addr 0x132]
				set temp [expr [expr $read_back & 0x01]]
				set fix_7 [expr $temp << 7]
				set fix [expr $fix_7 + $fix_6 + $fix_5 + $fix_4 + $fix_3_0]
				puts -nonewline "	Fix  : " 
				puts -nonewline [format 0x%x $fix]
				puts -nonewline "	Ipd  : "  
				set read_back [rd $m $base_addr 0x139]
				puts -nonewline  [expr [expr $read_back & 0x38] >> 3]
				puts ""
}}}}
close_service slave $m
}

puts {set_cdr_gear_all {dyn fix ipd}}
puts {example useage set_cdr_gear_all 31 3 3}
puts ""

###############################################################################
#Get CDR gear from all channels
###############################################################################
proc get_cdr_gear_all {} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {
		
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]


				# Print info
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				set identifier [rd $m $base_addr 0x200]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x $base_addr]	"
				puts -nonewline "	Dyn  : " 
				set read_back [rd $m $base_addr 0x137]
				puts -nonewline  [expr [expr $read_back & 0x7C] >> 2]

				set read_back [rd $m $base_addr 0x136]
				set fix_3_0 [expr $read_back & 0x0F]
				set read_back [rd $m $base_addr 0x134]
				set temp [expr [expr $read_back & 0x40] >> 6]
				set fix_4 [expr $temp << 4]
				set read_back [rd $m $base_addr 0x135]
				set temp	[expr [expr $read_back & 0x40] >> 6]	
				set fix_5 [expr $temp << 5]
				set read_back [rd $m $base_addr 0x132]
				set temp [expr [expr $read_back & 0x04] >> 2]	
				set fix_6 [expr $temp << 6]
				set read_back [rd $m $base_addr 0x132]
				set temp [expr [expr $read_back & 0x01]]
				set fix_7 [expr $temp << 7]
				set fix [expr $fix_7 + $fix_6 + $fix_5 + $fix_4 + $fix_3_0]
				puts -nonewline "	Fix  : " 
				puts -nonewline [format 0x%x $fix]
				puts -nonewline "	Ipd  : "  
				set read_back [rd $m $base_addr 0x139]
				puts -nonewline  [expr [expr $read_back & 0x38] >> 3]
				puts ""
}}}}
close_service slave $m
}
puts {get_cdr_gear_all {}}
puts {example useage get_cdr_gear_all}
puts ""


###############################################################################
#Set CDR gear to $dyn $fix  and $ipd for $phy
###############################################################################
proc set_cdr_gear_phy {phy index dyn fix ipd} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 
		
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 
		set base_addr [expr 0x1000 * $i]
		
		rmw $m $base_addr 0x137 0x7C [expr $dyn << 2]
		
		#Set fix 
		set fix_3_0 [expr $fix & 0x0F]
		set fix_4   [expr [expr $fix & 0x10] >> 4]
		set fix_5   [expr [expr $fix & 0x20] >> 5]
		set fix_6   [expr [expr $fix & 0x40] >> 6]
		set fix_7   [expr [expr $fix & 0x80] >> 7]
		rmw $m $base_addr 0x136 0x0F $fix_3_0
		rmw $m $base_addr 0x134 0x40 [expr $fix_4 << 6]
		rmw $m $base_addr 0x135 0x40 [expr $fix_5 << 6]
		rmw $m $base_addr 0x132 0x04 [expr $fix_6 << 2]
		rmw $m $base_addr 0x132 0x01 [expr $fix_7 << 0]
		
		#set IPD 0x139 bit [5:3]
		#rmw_channel( ((j<<10) + 0x139),0x38,0x18);	
		rmw $m $base_addr 0x139 0x38 [expr $ipd << 3]

				# Print info
				set identifier [rd $m $base_addr 0x200]
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x [expr $base_addr]]	"
				puts -nonewline "	Dyn  : " 
				set read_back [rd $m $base_addr 0x137]
				puts -nonewline  [expr [expr $read_back & 0x7C] >> 2]
				set read_back [rd $m $base_addr 0x136]
				set fix_3_0 [expr $read_back & 0x0F]
				set read_back [rd $m $base_addr 0x134]
				set temp [expr [expr $read_back & 0x40] >> 6]
				set fix_4 [expr $temp << 4]
				set read_back [rd $m $base_addr 0x135]
				set temp	[expr [expr $read_back & 0x40] >> 6]	
				set fix_5 [expr $temp << 5]
				set read_back [rd $m $base_addr 0x132]
				set temp [expr [expr $read_back & 0x04] >> 2]	
				set fix_6 [expr $temp << 6]
				set read_back [rd $m $base_addr 0x132]
				set temp [expr [expr $read_back & 0x01]]
				set fix_7 [expr $temp << 7]
				set fix [expr $fix_7 + $fix_6 + $fix_5 + $fix_4 + $fix_3_0]
				puts -nonewline "	Fix  : " 
				puts -nonewline [format 0x%x $fix]
				puts -nonewline "	Ipd  : "  
				set read_back [rd $m $base_addr 0x139]
				puts -nonewline  [expr [expr $read_back & 0x38] >> 3]
				puts ""
}}}}}
close_service slave $m
}

puts {set_cdr_gear_phy {phy index dyn fix ipd}}
puts {example useage set_cdr_gear_phy $phy 0 31 3 3}
puts ""

###############################################################################
#Get CDR gear settings from $phy 
###############################################################################
proc get_cdr_gear_phy {phy index} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 
		
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]

				# Print info
				set identifier [rd $m $base_addr 0x200]
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x [expr $base_addr ]]	"
				puts -nonewline "	Dyn  : " 
				set read_back [rd $m $base_addr 0x137]
				puts -nonewline  [expr [expr $read_back & 0x7C] >> 2]

				set read_back [rd $m $base_addr 0x136]
				set fix_3_0 [expr $read_back & 0x0F]
				set read_back [rd $m $base_addr 0x134]
				set temp [expr [expr $read_back & 0x40] >> 6]
				set fix_4 [expr $temp << 4]
				set read_back [rd $m $base_addr 0x135]
				set temp	[expr [expr $read_back & 0x40] >> 6]	
				set fix_5 [expr $temp << 5]
				set read_back [rd $m $base_addr 0x132]
				set temp [expr [expr $read_back & 0x04] >> 2]	
				set fix_6 [expr $temp << 6]
				set read_back [rd $m $base_addr 0x132]
				set temp [expr [expr $read_back & 0x01]]
				set fix_7 [expr $temp << 7]
				set fix [expr $fix_7 + $fix_6 + $fix_5 + $fix_4 + $fix_3_0]
				puts -nonewline "	Fix  : " 
				puts -nonewline [format 0x%x $fix]
				puts -nonewline "	Ipd  : "  
				set read_back [rd $m $base_addr 0x139]
				puts -nonewline  [expr [expr $read_back & 0x38] >> 3]
				puts ""
}}}}}
close_service slave $m
}

puts {get_cdr_gear_phy {phy index}}
puts {example useage get_cdr_gear_phy $phy 0}
puts ""

###############################################################################
#Set CDR Gear to $dyn $fix  and $ipd for $phy and $channel
###############################################################################
proc set_cdr_gear_channel {phy index channel dyn fix ipd} {

foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 

			set base_addr [expr [expr 0x1000 * $channel] ]


		#Set dyn to 31 : 0x137 bits [6:2]
		#rmw_channel( ((j<<10) + 0x137),0x7C,0x7C);
		
		rmw $m $base_addr 0x137 0x7C [expr $dyn << 2]
		
		#Set fix 
		set fix_3_0 [expr $fix & 0x0F]
		set fix_4   [expr [expr $fix & 0x10] >> 4]
		set fix_5   [expr [expr $fix & 0x20] >> 5]
		set fix_6   [expr [expr $fix & 0x40] >> 6]
		set fix_7   [expr [expr $fix & 0x80] >> 7]
		rmw $m $base_addr 0x136 0x0F $fix_3_0
		rmw $m $base_addr 0x134 0x40 [expr $fix_4 << 6]
		rmw $m $base_addr 0x135 0x40 [expr $fix_5 << 6]
		rmw $m $base_addr 0x132 0x04 [expr $fix_6 << 2]
		rmw $m $base_addr 0x132 0x01 [expr $fix_7 << 0]
		
		#set IPD 0x139 bit [5:3]
		#rmw_channel( ((j<<10) + 0x139),0x38,0x18);	
		rmw $m $base_addr 0x139 0x38 [expr $ipd << 3]


# Print info

set identifier [rd $m $base_addr 0x200]
set slave_addr [dict get $marker_info BASE_ADDRESS]
puts -nonewline "Slave: [format 0x%x $slave_addr]  "
puts -nonewline "channel  : $channel	"
puts -nonewline "base_addr : [format 0x%x [expr $base_addr ]]	"
puts -nonewline "	Dyn  : " 
set read_back [rd $m $base_addr 0x137]
puts -nonewline  [expr [expr $read_back & 0x7C] >> 2]
set read_back [rd $m $base_addr 0x136]
set fix_3_0 [expr $read_back & 0x0F]
set read_back [rd $m $base_addr 0x134]
set temp [expr [expr $read_back & 0x40] >> 6]
set fix_4 [expr $temp << 4]
set read_back [rd $m $base_addr 0x135]
set temp	[expr [expr $read_back & 0x40] >> 6]	
set fix_5 [expr $temp << 5]
set read_back [rd $m $base_addr 0x132]
set temp [expr [expr $read_back & 0x04] >> 2]	
set fix_6 [expr $temp << 6]
set read_back [rd $m $base_addr 0x132]
set temp [expr [expr $read_back & 0x01]]
set fix_7 [expr $temp << 7]
set fix [expr $fix_7 + $fix_6 + $fix_5 + $fix_4 + $fix_3_0]
puts -nonewline "	Fix  : " 
puts -nonewline [format 0x%x $fix]
puts -nonewline "	Ipd  : "  
set read_back [rd $m $base_addr 0x139]
puts -nonewline  [expr [expr $read_back & 0x38] >> 3]
puts ""
}}}}
close_service slave $m
}

puts {set_cdr_gear_channel {phy index channel  dyn fix ipd}}
puts {example useage set_cdr_gear_channel $phy 1 4 31 3 3}
puts ""

###############################################################################
#Do polarity inversion on rx for $phy and $channel
###############################################################################
proc rx_polarity_inv_on {phy index channel} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 

set base_addr [expr [expr 0x1000 * $channel] ]


		#Set rx_static_polarity_inversion bit 4 of 0xA
		rmw $m $base_addr 0xA 0x10 0x10
		
# Print info

set identifier [rd $m $base_addr 0x200]
set slave_addr [dict get $marker_info BASE_ADDRESS]
puts -nonewline "Slave: [format 0x%x $slave_addr]  "
puts -nonewline "channel  : $channel	"
puts -nonewline "base_addr : [format 0x%x [expr $base_addr ]]	"
puts -nonewline "	rx_static_polarity  : " 
set read_back [rd $m $base_addr 0xA]
puts -nonewline  [expr [expr $read_back & 0x10] >> 4]
}}}}
close_service slave $m
}

puts {rx_polarity_inv_on {phy index channel}}
puts {example useage rx_polarity_inv_on $phy 0 2}
puts ""

###############################################################################
#Disable polarity inversion on rx  for $phy and $channel
###############################################################################
proc rx_polarity_inv_off {phy index channel} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 

set base_addr [expr [expr 0x1000 * $channel] ]

		#Clear rx_static_polarity_inversion bit 4 of 0xA
		rmw $m $base_addr 0xA 0x10 0x00
		
# Print info
set identifier [rd $m $base_addr 0x200]
set slave_addr [dict get $marker_info BASE_ADDRESS]
puts -nonewline "Slave: [format 0x%x $slave_addr]  "
puts -nonewline "channel  : $channel	"
puts -nonewline "base_addr : [format 0x%x [expr $base_addr ]]	"
puts -nonewline "	rx_static_polarity  : " 
set read_back [rd $m $base_addr 0xA]
puts -nonewline  [expr [expr $read_back & 0x10] >> 4]
}}}}
close_service slave $m
}

puts {rx_polarity_inv_off {phy index channel}}
puts {example useage rx_polarity_inv_off $phy 0 2}
puts ""

proc status_dump_all {} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {
		
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]


				# JTAG and channel info
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				set identifier [rd $m $base_addr 0x200]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x $base_addr]	"
				puts ""

				# Identifier
				puts -nonewline "IP Identifier, 0x200, bit 7:0, " 
				puts [rd $m $base_addr 0x200]


				# Number of Channels
				puts -nonewline "Number of Channels, 0x210, bit 7:0, " 
				puts [rd $m $base_addr 0x210]

				# rx_is_lockedtodata
				puts -nonewline "rx_is_lockedtodata, 0x280, bit 0, " 
				puts [rd $m $base_addr 0x280]

				# rx_is_lockedtoref
				puts -nonewline "rx_is_lockedtoref, 0x280, bit 1, " 
				puts [rd $m $base_addr 0x280]


				# tx_cal_busy
				puts -nonewline "tx_cal_busy, 0x281, bit 0, " 
				puts [rd $m $base_addr 0x281]

				# rx_cal_busy
				puts -nonewline "rx_cal_busy, 0x281, bit 1, " 
				puts [rd $m $base_addr 0x281]

				# vod
				puts -nonewline "vod, 0x109, bit 4:0, " 
				puts -nonewline [rd $m $base_addr 0x109]
				puts -nonewline ", dec=,  "
				puts [rd_mask_dec $m $base_addr 0x109 0x1f]

				# TX CGB regulator voltage boost (vccdreg_output)
				puts -nonewline "CGB regulator voltage boost, 0x113, bit 4:0, " 
				puts [rd $m $base_addr 0x113]

				# TX compensation 
				#register 0x109 0x109[5]
				puts -nonewline "TX Compensation, 0x109, bit 5, " 
				puts [rd $m $base_addr 0x109]
				
				# pre-emp
				puts -nonewline "pre-emp 1st post tap, 0x105, bit 4:0, " 
				puts -nonewline [rd $m $base_addr 0x105]
				puts -nonewline ", dec=, "
				puts [rd_mask_dec $m $base_addr 0x105 0x1f]

				puts -nonewline "pre-emp 1st post tap sign, 0x105, bit 6, " 
				puts [rd $m $base_addr 0x105]


				puts -nonewline "pre-emp 2nd post tap, 0x106, bit 3:0, " 
				puts -nonewline [rd $m $base_addr 0x106]
				puts -nonewline ", dec=, "
				puts [rd_mask_dec $m $base_addr 0x106 0x0f]

				puts -nonewline "pre-emp 2nd post tap sign, 0x106, bit 5, " 
				puts [rd $m $base_addr 0x106]


				puts -nonewline "pre-emp 1st pre tap, 0x107, bit 4:0, " 
				puts -nonewline [rd $m $base_addr 0x107]
				puts -nonewline ", dec=, "
				puts [rd_mask_dec $m $base_addr 0x107 0x1f]

				puts -nonewline "pre-emp 1st pre tap sign, 0x107, bit 5, " 
				puts [rd $m $base_addr 0x107]


				puts -nonewline "pre-emp 2nd pre tap, 0x108, bit 2:0, " 
				puts -nonewline [rd $m $base_addr 0x108]
				puts -nonewline ", dec=, "
				puts [rd_mask_dec $m $base_addr 0x108 0x07]

				puts -nonewline "pre-emp 2nd pre tap sign, 0x108, bit 4, " 
				puts [rd $m $base_addr 0x108]


				# dc gain
				puts -nonewline "DC Gain, 0x11C, bit 3:0, " 
				puts [rd $m $base_addr 0x11C]
				puts -nonewline "DC Gain, 0x11A, bit 7:0, " 
				puts [rd $m $base_addr 0x11A]


				# ac gain - ctle
				#   bit [3]=0 for 4 stage mode which is high gain CTLE
				puts -nonewline "CTLE eq path, 0x11B, bit 3, " 
				puts [rd $m $base_addr 0x11B]

				puts -nonewline "CTLE AC gain, 0x167, bit 5:1, " 
				puts [rd $m $base_addr 0x167]

				puts -nonewline "CTLE Adapt bypass, 0x167, bit 0, " 
				puts [rd $m $base_addr 0x167]
				

				# VGA bandwidth and eq bandwidth
				puts -nonewline "VGA bandwidth, 0x11F, bit 7:6, "
				puts [rd $m $base_addr 0x11F]
				puts -nonewline "EQ bandwidth, 0x11F, bit 5:4, "
				puts [rd $m $base_addr 0x11F] 

				# VGA settings
				#0x160[0] (adp_vga_bypass)   
				puts -nonewline "VGA bypass adp_vga_bypass, 0x160, bit 0, "
				puts [rd $m $base_addr 0x160]

				#0x166[0] (adp_1s_ctle_bypass)   
				puts -nonewline "VGA adp_1s_ctle_bypass, 0x166, bit 0, "
				puts [rd $m $base_addr 0x166]
				
				#0x148[3] (adp_vga_en)   VGA Adaptation enable
				puts -nonewline "VGA adaptation enable, 0x148, bit 3, "
				puts [rd $m $base_addr 0x148]
				
				puts -nonewline "VGA gain, 0x160, bit 3:1, "
				puts [rd $m $base_addr 0x160]

				
				# CDR
				puts -nonewline "CDR dyn, 0x137, bit 6:2, "
				puts [rd $m $base_addr 0x137]

				puts -nonewline "CDR fix, 0x134, bit 6, "
				puts [rd $m $base_addr 0x134]

				puts -nonewline "CDR fix, 0x136, bit 3:0, "
				puts [rd $m $base_addr 0x136]

				puts -nonewline "R_pd, 0x135, bit 3:2, "
				puts [rd $m $base_addr 0x135]

				puts -nonewline "I_pd, 0x139, bit 5:3, "
				puts [rd $m $base_addr 0x139]

				puts -nonewline "M counter, 0x13B, bit 7:0, "
				puts [rd $m $base_addr 0x13B]


				# CDR Lock to ref loop
				puts -nonewline "L_pfd, 0x13A, bit 2:0, "
				puts [rd $m $base_addr 0x13A]

				puts -nonewline "R_pfd, 0x135, bit 1:0, "
				puts [rd $m $base_addr 0x135]

				puts -nonewline "I_pfd, 0x139, bit 2:0, "
				puts [rd $m $base_addr 0x139]
				
				

				# Serial Loopback
				puts -nonewline "Serial loopback, 0x2e1, bit 0, " 
				puts [rd $m $base_addr 0x2e1]

				# Reverse Serial Loopback
				puts -nonewline "Reverse Serial loopback, 0x132, bit 5:4, " 
				puts [rd $m $base_addr 0x132]



				puts ""
				puts ""
}}}}
close_service slave $m
}

puts {status_dump_all{}}
puts {example useage status_dump_all}
puts ""



								 
proc status_dump_phy {phy index } {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 
		
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]

				 				

				# Print info
				set identifier [rd $m $base_addr 0x200]
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x [expr $base_addr ]]	"
				puts ""

				# Identifier
				puts -nonewline "IP Identifier, 0x200, bit 7:0, " 
				puts [rd $m $base_addr 0x200]


				# Number of Channels
				puts -nonewline "Number of Channels, 0x210, bit 7:0, " 
				puts [rd $m $base_addr 0x210]

				# rx_is_lockedtodata
				puts -nonewline "rx_is_lockedtodata, 0x280, bit 0, " 
				puts [rd $m $base_addr 0x280]

				# rx_is_lockedtoref
				puts -nonewline "rx_is_lockedtoref, 0x280, bit 1, " 
				puts [rd $m $base_addr 0x280]


				# tx_cal_busy
				puts -nonewline "tx_cal_busy, 0x281, bit 0, " 
				puts [rd $m $base_addr 0x281]

				# rx_cal_busy
				puts -nonewline "rx_cal_busy, 0x281, bit 1, " 
				puts [rd $m $base_addr 0x281]

				# vod
				puts -nonewline "vod, 0x109, bit 4:0, " 
				puts -nonewline [rd $m $base_addr 0x109]
				puts -nonewline ", dec=,  "
				puts [rd_mask_dec $m $base_addr 0x109 0x1f]

				# TX CGB regulator voltage boost (vccdreg_output)
				puts -nonewline "CGB regulator voltage boost, 0x113, bit 4:0, " 
				puts [rd $m $base_addr 0x113]

				# TX compensation 
				#register 0x109 0x109[5]
				puts -nonewline "TX Compensation, 0x109, bit 5, " 
				puts [rd $m $base_addr 0x109]
				
				# pre-emp
				puts -nonewline "pre-emp 1st post tap, 0x105, bit 4:0, " 
				puts -nonewline [rd $m $base_addr 0x105]
				puts -nonewline ", dec=, "
				puts [rd_mask_dec $m $base_addr 0x105 0x1f]

				puts -nonewline "pre-emp 1st post tap sign, 0x105, bit 6, " 
				puts [rd $m $base_addr 0x105]


				puts -nonewline "pre-emp 2nd post tap, 0x106, bit 3:0, " 
				puts -nonewline [rd $m $base_addr 0x106]
				puts -nonewline ", dec=, "
				puts [rd_mask_dec $m $base_addr 0x106 0x0f]

				puts -nonewline "pre-emp 2nd post tap sign, 0x106, bit 5, " 
				puts [rd $m $base_addr 0x106]


				puts -nonewline "pre-emp 1st pre tap, 0x107, bit 4:0, " 
				puts -nonewline [rd $m $base_addr 0x107]
				puts -nonewline ", dec=, "
				puts [rd_mask_dec $m $base_addr 0x107 0x1f]

				puts -nonewline "pre-emp 1st pre tap sign, 0x107, bit 5, " 
				puts [rd $m $base_addr 0x107]


				puts -nonewline "pre-emp 2nd pre tap, 0x108, bit 2:0, " 
				puts -nonewline [rd $m $base_addr 0x108]
				puts -nonewline ", dec=, "
				puts [rd_mask_dec $m $base_addr 0x108 0x07]

				puts -nonewline "pre-emp 2nd pre tap sign, 0x108, bit 4, " 
				puts [rd $m $base_addr 0x108]


				# dc gain
				puts -nonewline "DC Gain, 0x11C, bit 3:0, " 
				puts [rd $m $base_addr 0x11C]
				puts -nonewline "DC Gain, 0x11A, bit 7:0, " 
				puts [rd $m $base_addr 0x11A]


				# ac gain - ctle
				#   bit [3]=0 for 4 stage mode which is high gain CTLE
				puts -nonewline "CTLE eq path, 0x11B, bit 3, " 
				puts [rd $m $base_addr 0x11B]

				puts -nonewline "CTLE AC gain, 0x167, bit 5:1, " 
				puts [rd $m $base_addr 0x167]

				puts -nonewline "CTLE Adapt bypass, 0x167, bit 0, " 
				puts [rd $m $base_addr 0x167]
				

				# VGA bandwidth and eq bandwidth
				puts -nonewline "VGA bandwidth, 0x11F, bit 7:6, "
				puts [rd $m $base_addr 0x11F]
				puts -nonewline "EQ bandwidth, 0x11F, bit 5:4, "
				puts [rd $m $base_addr 0x11F] 

				# VGA settings
				#0x160[0] (adp_vga_bypass)   
				puts -nonewline "VGA bypass adp_vga_bypass, 0x160, bit 0, "
				puts [rd $m $base_addr 0x160]

				#0x166[0] (adp_1s_ctle_bypass)   
				puts -nonewline "VGA adp_1s_ctle_bypass, 0x166, bit 0, "
				puts [rd $m $base_addr 0x166]
				
				#0x148[3] (adp_vga_en)   VGA Adaptation enable
				puts -nonewline "VGA adaptation enable, 0x148, bit 3, "
				puts [rd $m $base_addr 0x148]
				
				puts -nonewline "VGA gain, 0x160, bit 3:1, "
				puts [rd $m $base_addr 0x160]

				
				# CDR
				puts -nonewline "CDR dyn, 0x137, bit 6:2, "
				puts [rd $m $base_addr 0x137]

				puts -nonewline "CDR fix, 0x134, bit 6, "
				puts [rd $m $base_addr 0x134]

				puts -nonewline "CDR fix, 0x136, bit 3:0, "
				puts [rd $m $base_addr 0x136]

				puts -nonewline "R_pd, 0x135, bit 3:2, "
				puts [rd $m $base_addr 0x135]

				puts -nonewline "I_pd, 0x139, bit 5:3, "
				puts [rd $m $base_addr 0x139]

				puts -nonewline "M counter, 0x13B, bit 7:0, "
				puts [rd $m $base_addr 0x13B]


				# CDR Lock to ref loop
				puts -nonewline "L_pfd, 0x13A, bit 2:0, "
				puts [rd $m $base_addr 0x13A]

				puts -nonewline "R_pfd, 0x135, bit 1:0, "
				puts [rd $m $base_addr 0x135]

				puts -nonewline "I_pfd, 0x139, bit 2:0, "
				puts [rd $m $base_addr 0x139]
				
				

				# Serial Loopback
				puts -nonewline "Serial loopback, 0x2e1, bit 0, " 
				puts [rd $m $base_addr 0x2e1]

				# Reverse Serial Loopback
				puts -nonewline "Reverse Serial loopback, 0x132, bit 5:4, " 
				puts [rd $m $base_addr 0x132]



				puts ""
				puts ""
}}}}}
close_service slave $m
}

puts {status_dump_phy{phy index}}
puts {example useage status_dump_phy $phy 1}
puts ""


		

proc status_dump_channel {phy index channel} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 

set base_addr [expr [expr 0x1000 * $channel] ]


				 				

				# Print info
set identifier [rd $m $base_addr 0x200]
set slave_addr [dict get $marker_info BASE_ADDRESS]
puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $channel	"
				puts -nonewline "base_addr : [format 0x%x [expr $base_addr]]	"
				puts ""

				# Identifier
				puts -nonewline "IP Identifier, 0x200, bit 7:0, " 
				puts [rd $m $base_addr 0x200]


				# Number of Channels
				puts -nonewline "Number of Channels, 0x210, bit 7:0, " 
				puts [rd $m $base_addr 0x210]

				# rx_is_lockedtodata
				puts -nonewline "rx_is_lockedtodata, 0x280, bit 0, " 
				puts [rd $m $base_addr 0x280]

				# rx_is_lockedtoref
				puts -nonewline "rx_is_lockedtoref, 0x280, bit 1, " 
				puts [rd $m $base_addr 0x280]


				# tx_cal_busy
				puts -nonewline "tx_cal_busy, 0x281, bit 0, " 
				puts [rd $m $base_addr 0x281]

				# rx_cal_busy
				puts -nonewline "rx_cal_busy, 0x281, bit 1, " 
				puts [rd $m $base_addr 0x281]

				# vod
				puts -nonewline "vod, 0x109, bit 4:0, " 
				puts -nonewline [rd $m $base_addr 0x109]
				puts -nonewline ", dec=,  "
				puts [rd_mask_dec $m $base_addr 0x109 0x1f]

				# TX CGB regulator voltage boost (vccdreg_output)
				puts -nonewline "CGB regulator voltage boost, 0x113, bit 4:0, " 
				puts [rd $m $base_addr 0x113]

				# TX compensation 
				#register 0x109 0x109[5]
				puts -nonewline "TX Compensation, 0x109, bit 5, " 
				puts [rd $m $base_addr 0x109]
				
				# pre-emp
				puts -nonewline "pre-emp 1st post tap, 0x105, bit 4:0, " 
				puts -nonewline [rd $m $base_addr 0x105]
				puts -nonewline ", dec=, "
				puts [rd_mask_dec $m $base_addr 0x105 0x1f]

				puts -nonewline "pre-emp 1st post tap sign, 0x105, bit 6, " 
				puts [rd $m $base_addr 0x105]


				puts -nonewline "pre-emp 2nd post tap, 0x106, bit 3:0, " 
				puts -nonewline [rd $m $base_addr 0x106]
				puts -nonewline ", dec=, "
				puts [rd_mask_dec $m $base_addr 0x106 0x0f]

				puts -nonewline "pre-emp 2nd post tap sign, 0x106, bit 5, " 
				puts [rd $m $base_addr 0x106]


				puts -nonewline "pre-emp 1st pre tap, 0x107, bit 4:0, " 
				puts -nonewline [rd $m $base_addr 0x107]
				puts -nonewline ", dec=, "
				puts [rd_mask_dec $m $base_addr 0x107 0x1f]

				puts -nonewline "pre-emp 1st pre tap sign, 0x107, bit 5, " 
				puts [rd $m $base_addr 0x107]


				puts -nonewline "pre-emp 2nd pre tap, 0x108, bit 2:0, " 
				puts -nonewline [rd $m $base_addr 0x108]
				puts -nonewline ", dec=, "
				puts [rd_mask_dec $m $base_addr 0x108 0x07]

				puts -nonewline "pre-emp 2nd pre tap sign, 0x108, bit 4, " 
				puts [rd $m $base_addr 0x108]


				# dc gain
				puts -nonewline "DC Gain, 0x11C, bit 3:0, " 
				puts [rd $m $base_addr 0x11C]
				puts -nonewline "DC Gain, 0x11A, bit 7:0, " 
				puts [rd $m $base_addr 0x11A]


				# ac gain - ctle
				#   bit [3]=0 for 4 stage mode which is high gain CTLE
				puts -nonewline "CTLE eq path, 0x11B, bit 3, " 
				puts [rd $m $base_addr 0x11B]

				puts -nonewline "CTLE AC gain, 0x167, bit 5:1, " 
				puts [rd $m $base_addr 0x167]

				puts -nonewline "CTLE Adapt bypass, 0x167, bit 0, " 
				puts [rd $m $base_addr 0x167]
				

				# VGA bandwidth and eq bandwidth
				puts -nonewline "VGA bandwidth, 0x11F, bit 7:6, "
				puts [rd $m $base_addr 0x11F]
				puts -nonewline "EQ bandwidth, 0x11F, bit 5:4, "
				puts [rd $m $base_addr 0x11F] 

				# VGA settings
				#0x160[0] (adp_vga_bypass)   
				puts -nonewline "VGA bypass adp_vga_bypass, 0x160, bit 0, "
				puts [rd $m $base_addr 0x160]

				#0x166[0] (adp_1s_ctle_bypass)   
				puts -nonewline "VGA adp_1s_ctle_bypass, 0x166, bit 0, "
				puts [rd $m $base_addr 0x166]
				
				#0x148[3] (adp_vga_en)   VGA Adaptation enable
				puts -nonewline "VGA adaptation enable, 0x148, bit 3, "
				puts [rd $m $base_addr 0x148]
				
				puts -nonewline "VGA gain, 0x160, bit 3:1, "
				puts [rd $m $base_addr 0x160]

				
				# CDR
				puts -nonewline "CDR dyn, 0x137, bit 6:2, "
				puts [rd $m $base_addr 0x137]

				puts -nonewline "CDR fix, 0x134, bit 6, "
				puts [rd $m $base_addr 0x134]

				puts -nonewline "CDR fix, 0x136, bit 3:0, "
				puts [rd $m $base_addr 0x136]

				puts -nonewline "R_pd, 0x135, bit 3:2, "
				puts [rd $m $base_addr 0x135]

				puts -nonewline "I_pd, 0x139, bit 5:3, "
				puts [rd $m $base_addr 0x139]

				puts -nonewline "M counter, 0x13B, bit 7:0, "
				puts [rd $m $base_addr 0x13B]


				# CDR Lock to ref loop
				puts -nonewline "L_pfd, 0x13A, bit 2:0, "
				puts [rd $m $base_addr 0x13A]

				puts -nonewline "R_pfd, 0x135, bit 1:0, "
				puts [rd $m $base_addr 0x135]

				puts -nonewline "I_pfd, 0x139, bit 2:0, "
				puts [rd $m $base_addr 0x139]
				
				

				# Serial Loopback
				puts -nonewline "Serial loopback, 0x2e1, bit 0, " 
				puts [rd $m $base_addr 0x2e1]

				# Reverse Serial Loopback
				puts -nonewline "Reverse Serial loopback, 0x132, bit 5:4, " 
				puts [rd $m $base_addr 0x132]



				puts ""
				puts ""
}}}}
close_service slave $m
}

puts {status_dump_channel{phy index channel}}
puts {example useage status_dump_channel $phy 1 3}
puts ""

proc compare_xcvr_channels {phy index ch1 ch2 start_addr stop_addr} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 
		
	set base_addr1 [expr [expr 0x1000 * $ch1] ]
	set base_addr2 [expr [expr 0x1000 * $ch2] ]
	
set addr $start_addr

	for {set i 0} {$i < [expr {$stop_addr - $start_addr}]} {incr i} {
		set d1 [rd $m $base_addr1 $addr]
		set d2 [rd $m $base_addr2 $addr]
		#puts $d1
		#puts $d2
		if {[expr {$d1 ne $d2}] } {
			puts [format "ADDR: 0x%03x --> ch%d:  0x%02x   /   ch%d: 0x%02x" $addr $ch1 $d1 $ch2 $d2]  
			}
		incr addr
	}
}}}}
close_service slave $m
}

puts {compare_xcvr_channels {phy index ch1 ch2 start_addr stop_addr}}
puts {example useage compare_xcvr_channels $phy 0 3 5 0x100 0x250}
puts ""


###############################################################################
#Control serial loopback for all channels
###############################################################################
proc set_serial_loop_all {loopback} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {
		
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]


		#Bit 0 of 0x2E1.
		
		rmw $m $base_addr 0x2E1 0x01 $loopback
		


				# Print info
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				set identifier [rd $m $base_addr 0x200]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x $base_addr]	"
				puts -nonewline "	Loopback  : " 
				set read_back [rd $m $base_addr 0x2E1]
				puts -nonewline  [expr [expr $read_back & 0x01] ]
				puts ""
}}}}
close_service slave $m
}

puts {set_serial_loop_all {loopback}}
puts {example useage set_serial_loop_all 1}
puts ""


###############################################################################
#Control serial loopback for a specific phy
###############################################################################
proc set_serial_loop_phy {phy index loopback} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 
		
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]
		#Bit 0 of 0x2E1.
		
			rmw $m $base_addr 0x2E1 0x01 $loopback
		

		

				# Print info			

				set identifier [rd $m $base_addr 0x200]
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x [expr $base_addr ]]	"
				puts -nonewline "	Loopback  : " 
				set read_back [rd $m $base_addr 0x2E1]
				puts -nonewline  [expr [expr $read_back & 0x01] ]
				puts ""
}}}}}
close_service slave $m
}

puts {set_serial_loop_phy {phy index loopback}}
puts {example useage set_serial_loop_phy $phy 1 1}
puts ""



###############################################################################
#Enable Reverse Serial loopback (post-CDR)to $channel in $phy
###############################################################################
proc enable_rev_serial_post {phy index channel} {

foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 

set base_addr [expr [expr 0x1000 * $channel] ]

				
#-----------------------enable reverse serial loopback (post-CDR)-----------------------#
#register 0x13C[7]
rmw $m $base_addr 0x13C 0x80 0x80
 
#register 0x137[7]
rmw $m $base_addr 0x137 0x80 0x80
 
#register 0x132[5:4]
rmw $m $base_addr 0x132 0x30 0x10
 
#register 0x142[4] (0 for post-CDR)
rmw $m $base_addr 0x142 0x10 0x00
 
#register 0x11D[0] (0 for post-CDR)
rmw $m $base_addr 0x11D 0x01 0x00



# Print info
set identifier [rd $m $base_addr 0x200]
set slave_addr [dict get $marker_info BASE_ADDRESS]
puts -nonewline "Slave: [format 0x%x $slave_addr]  "
puts -nonewline "Identifier: $identifier  "
puts -nonewline "channel  : $channel	"
puts -nonewline "base_addr : [format 0x%x [expr $base_addr ]]	"
puts -nonewline "	Post CDR loopback enabled  " 
puts ""
}}}}
close_service slave $m
}

puts {enable_rev_serial_post {phy index channel}}
puts {example useage enable_rev_serial_post $phy 0 1}
puts ""

###############################################################################
#Enable Reverse Serial loopback (pre-CDR)to $channel in $phy
###############################################################################
proc enable_rev_serial_pre {phy index channel} {

foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 

set base_addr [expr [expr 0x1000 * $channel] ]

				
# -----------------------enable reverse serial loopback (pre-CDR)-----------------------#
#register 0x137[7]
rmw $m $base_addr 0x137 0x80 0x80
 
#register 0x13C[7]
rmw $m $base_addr 0x13C 0x80 0x80
 
#register 0x132[5:4]
rmw $m $base_addr 0x132 0x30 0x10
 
#register 0x142[4] (1 for pre-CDR)
rmw $m $base_addr 0x142 0x10 0x10
 
#register 0x11D[0] (1 for pre-CDR)
rmw $m $base_addr 0x11D 0x01 0x01



# Print info
set identifier [rd $m $base_addr 0x200]
set slave_addr [dict get $marker_info BASE_ADDRESS]
puts -nonewline "Slave: [format 0x%x $slave_addr]  "
puts -nonewline "Identifier: $identifier  "
puts -nonewline "channel  : $channel	"
puts -nonewline "base_addr : [format 0x%x [expr $base_addr ]]	"
puts -nonewline "	Pre CDR loopback enabled  " 
puts ""
}}}}
close_service slave $m
}
puts {enable_rev_serial_pre {phy index channel}}
puts {example useage enable_rev_serial_pre $phy 0 1}
puts ""

###############################################################################
#Disable Reverse Serial loopback (pre or post CDR) to $channel in $phy
###############################################################################
proc disable_rev_serial {phy index channel} {

foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 

set base_addr [expr [expr 0x1000 * $channel] ]
			
# -----------------------disable reverse serial loopback (pre-CDR or post-cdr)-----------#
#register 0x137[7]
rmw $m $base_addr 0x137 0x80 0x00
 
#register 0x13C[7]
rmw $m $base_addr 0x13C 0x80 0x00
 
#register 0x132[5:4]
rmw $m $base_addr 0x132 0x30 0x00
 
#register 0x142[4]
rmw $m $base_addr 0x142 0x10 0x00
 
#register 0x11D[0]
rmw $m $base_addr 0x11D 0x01 0x00




# Print info

set identifier [rd $m $base_addr 0x200]
set slave_addr [dict get $marker_info BASE_ADDRESS]
puts -nonewline "Slave: [format 0x%x $slave_addr]  "
puts -nonewline "Identifier: $identifier  "
puts -nonewline "channel  : $channel	"
puts -nonewline "base_addr : [format 0x%x [expr $base_addr ]]	"
puts -nonewline "Reverse Serial loopback disabled  " 
puts ""
}}}}
close_service slave $m
}

puts {disable_rev_serial {phy index channel}}
puts {example useage disable_rev_serial $phy 0 1}
puts ""



###############################################################################
#Enable Continous DFE only on a specific channel in a PHY
###############################################################################

proc continuous_dfe_only {phy index channel} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {
			
		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 

				set base_addr [expr 0x1000 * $channel]
		
				rmw $m $base_addr 0x120 0x0F 0x02
				rmw $m $base_addr 0x149 0x0F 0x08
				rmw $m $base_addr 0x148 0x20 0x20
				rmw $m $base_addr 0x14A 0x07 0x02
				rmw $m $base_addr 0x14A 0x18 0x18
				rmw $m $base_addr 0x148 0x1F 0x1F
				rmw $m $base_addr 0x15B 0x01 0x00
				rmw $m $base_addr 0x15B 0x04 0x04
				rmw $m $base_addr 0x15E 0x01 0x00
				rmw $m $base_addr 0x160 0x01 0x01
				rmw $m $base_addr 0x166 0x01 0x01
				rmw $m $base_addr 0x167 0x01 0x01
				rmw $m $base_addr 0x14B 0x01 0x00
				rmw $m $base_addr 0x15D 0x1F 0x15
				rmw $m $base_addr 0x15F 0x01 0x01
				rmw $m $base_addr 0x15F 0x0E 0x08
				rmw $m $base_addr 0x15F 0x10 0x00
				rmw $m $base_addr 0x15C 0x38 0x28
				rmw $m $base_addr 0x161 0x01 0x01
				rmw $m $base_addr 0x161 0x08 0x00
				rmw $m $base_addr 0x161 0x10 0x00
				rmw $m $base_addr 0x161 0x20 0x00
				rmw $m $base_addr 0x162 0xE0 0x60
				rmw $m $base_addr 0x163 0x01 0x00
				rmw $m $base_addr 0x163 0xE0 0x00
				rmw $m $base_addr 0x162 0x1F 0x00
				rmw $m $base_addr 0x14D 0x07 0x00
				rmw $m $base_addr 0x14D 0x38 0x38
				rmw $m $base_addr 0x14B 0x80 0x00
				rmw $m $base_addr 0x15B 0x10 0x00
				rmw $m $base_addr 0x123 0x01 0x01
				rmw $m $base_addr 0x123 0x02 0x02
				rmw $m $base_addr 0x123 0x08 0x08
				rmw $m $base_addr 0x149 0x10 0x10
				rmw $m $base_addr 0x149 0x40 0x00
				rmw $m $base_addr 0x149 0x40 0x40
				rmw $m $base_addr 0x149 0x20 0x00
				rmw $m $base_addr 0x149 0x20 0x20		
				
				# Print info
				set identifier [rd $m 0x0 0x200]
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $channel	"
				puts -nonewline "base_addr : [format 0x%x $base_addr]	"
				puts -nonewline " continuous dfe enabled " 
				puts ""
	close_service slave $m
}}}}
close_service slave $m
}

puts {continuous_dfe_only {phy index channel}}
puts {example continuous_dfe_only $phy 0 3}
puts ""

###############################################################################
#Enable triggered CTLE  on a specific channel in a PHY
###############################################################################
proc triggered_ctle {phy index channel} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {
			
		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 


				set base_addr [expr 0x1000 * $channel]
		
# ************ Steps to perform Triggered CLTE Only Adaptation ************************************

rmw $m $base_addr 0x120 0x0F 0x02
rmw $m $base_addr 0x149 0x0F 0x08
rmw $m $base_addr 0x148 0x20 0x20
rmw $m $base_addr 0x14A 0x07 0x02
rmw $m $base_addr 0x14A 0x18 0x18
rmw $m $base_addr 0x148 0x1F 0x1C
rmw $m $base_addr 0x15B 0x01 0x01
rmw $m $base_addr 0x15B 0x04 0x04
rmw $m $base_addr 0x15E 0x01 0x00
rmw $m $base_addr 0x160 0x01 0x00
rmw $m $base_addr 0x166 0x01 0x00
rmw $m $base_addr 0x167 0x01 0x00
rmw $m $base_addr 0x14B 0x01 0x00
rmw $m $base_addr 0x15D 0x1F 0x15
rmw $m $base_addr 0x15F 0x01 0x01
rmw $m $base_addr 0x15F 0x0E 0x08
rmw $m $base_addr 0x15F 0x10 0x00
rmw $m $base_addr 0x15C 0x38 0x28
rmw $m $base_addr 0x161 0x01 0x01
rmw $m $base_addr 0x161 0x08 0x00
rmw $m $base_addr 0x161 0x10 0x00
rmw $m $base_addr 0x161 0x20 0x00
rmw $m $base_addr 0x162 0xE0 0x60
rmw $m $base_addr 0x163 0x01 0x00
rmw $m $base_addr 0x163 0xE0 0xE0
rmw $m $base_addr 0x162 0x1F 0x00
rmw $m $base_addr 0x14D 0x07 0x00
rmw $m $base_addr 0x14D 0x38 0x38
rmw $m $base_addr 0x14B 0x80 0x80
rmw $m $base_addr 0x15B 0x10 0x10
rmw $m $base_addr 0x123 0x01 0x01
rmw $m $base_addr 0x123 0x02 0x02
rmw $m $base_addr 0x123 0x08 0x00
rmw $m $base_addr 0x149 0x10 0x10
rmw $m $base_addr 0x149 0x40 0x00
rmw $m $base_addr 0x149 0x40 0x40
rmw $m $base_addr 0x149 0x20 0x00
rmw $m $base_addr 0x149 0x20 0x20
	
				
				# Print info
				set identifier [rd $m 0x0 0x200]
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $channel	"
				puts -nonewline "base_addr : [format 0x%x $base_addr]	"
				puts -nonewline " triggered ctle enabled " 
				puts ""
	close_service slave $m
}}}}
close_service slave $m
}

puts {triggered_ctle {phy index channel}}
puts {example triggered_ctle $phy 1 3}
puts ""



###############################################################################
#Get converged CTLE value from all channels
###############################################################################
proc get_converged_all {} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {
		
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]

			# To read the converged CTLE/VGA/adapt done values for each lane values from the PMA testbus

# 6'b011011 : status_bus = {1'b0,vga_coef_ctle[2],vga_coef_ctle[1],vga_coef_ctle[0],ctle_coef_1s_ctle[3],ctle_coef_1s_ctle[2],ctle_coef_1s_ctle[1],ctle_coef_1s_ctle[0]};
# 6'b011100 : status_bus = {dfe_vref_sign_sel,dfe_spec_sign_sel,dfe_spec_disable,vref_coef_ctle[4],vref_coef_ctle[3],vref_coef_ctle[2],vref_coef_ctle[1],vref_coef_ctle[0]};
				


				# Print info
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				set identifier [rd $m $base_addr 0x200]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $i	"
				puts -nonewline "base_addr : [format 0x%x $base_addr]	"

				# set status_bus to read out vga and ctle coefficients
				rmw $m $base_addr 0x14C 0x3F 0x1B

				puts -nonewline "	Converged CTLE : " 
				set read_back [rd $m $base_addr 0x177]
				#set temp  [expr $read_back & 0x0F]
				#set ctle_dedoded [decode_ctle_converged_value $temp]
				puts -nonewline  [expr $read_back & 0x0F]

				puts -nonewline "	Converged VGA  : " 
				puts -nonewline  [expr [expr $read_back & 0x70] >> 4]

				# set status_bus to read out vref 
				rmw $m $base_addr 0x14C 0x3F 0x1C

				puts -nonewline "	Vref  : " 
				set read_back [rd $m $base_addr 0x177]
				puts -nonewline  [format 0x%2x [expr $read_back & 0x1F]]
				
				# set status_bus to read out adapt done
				rmw $m $base_addr 0x14C 0x3F 0x2E

				puts -nonewline "     Adapt Done : " 
				set read_back [rd $m $base_addr 0x177]
				puts -nonewline  [format 0x%x [expr $read_back & 0xFF]]

				puts ""
}}}}
close_service slave $m
}

puts {get_converged_all {}}
puts {example useage get_converged_all}
puts ""

#proc decode_ctle_converged_value {ctle_converge} {
#    switch $ctle_converge{
#        0  { return [expr 0x0] }
#        1  { return [expr 0x2] }
#        2  { return [expr 0x4] }
#        3  { return [expr 0x6] }
#        4  { return [expr 0x8] }
#        5  { return [expr 0xA] }
#        6  { return [expr 0xC] }
#        7  { return [expr 0xE] }
#        8  { return [expr 0x10] }
#        9  { return [expr 0x12] }
#        10 { return [expr 0x14] }
#        11 { return [expr 0x16] }
#        12 { return [expr 0x18] }
#        13 { return [expr 0x1A] }
#        14 { return [expr 0x1B] }
#        15 { return [expr 0x1F] }
#    }
#}

###############################################################################
#Show dfe taps of all channels
###############################################################################
proc show_dfe_taps_all {} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {
		
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]


#-----------------------DFE readback-----------------------#
#{0x71,0x01,0x1E,0x00}
#address 0x171 for top level testbus_sel - 0x3 <<1 for DFE
#rmw $m $base_addr 0x171 0x1e 0x06
#dfe outputs
rmw $m $base_addr 0x171 0x1e 0x14

#Address: 0x149[5:0] select bits 
#tap 1 = 011_1_1_0
#tap 2 = 011_1_1_1
#tap 3 = 100_0_0_0
#tap 4 = 100_0_0_1
#tap 5 = 100_0_1_0
#tap 6 = 100_0_1_1
#tap 7 = 100_1_0_0

#0x130 [3:0] sel_probe_tstmux
#probe_fxtap1_coeff	4'b0001
#probe_fxtap2_coeff	4'b0010
#probe_fxtap3_coeff	4'b0011
#probe_fxtap4_coeff	4'b0100
#probe_fxtap5_coeff	4'b0101
#probe_fxtap6_coeff	4'b0110
#probe_fxtap7_coeff	4'b0111
#probe_fltap1_coeff 4'b1011
#probe_fltap2_coeff 4'b1100
#probe_fltap3_coeff 4'b1101
#probe_fltap4_coeff 4'b1110


rmw $m $base_addr 0x130 0x0F 0x1
set tap1 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x130 0x0F 0x2
set tap2 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x130 0x0F 0x3
set tap3 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x130 0x0F 0x4
set tap4 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x130 0x0F 0x5
set tap5 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x130 0x0F 0x6
set tap6 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x130 0x0F 0x7
set tap7 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]


#floating tap readout is different?
#dfe outputs
rmw $m $base_addr 0x171 0x1e 0x16

rmw $m $base_addr 0x14C 0x3f 0x25
#rmw $m $base_addr 0x130 0x0F 0xB
set fltap1 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x14C 0x3f 0x26
#rmw $m $base_addr 0x130 0x0F 0xC
set fltap2 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x14C 0x3F 0x27
#rmw $m $base_addr 0x130 0x0F 0xD
set fltap3 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x14C 0x3F 0x28
#rmw $m $base_addr 0x130 0x0F 0xE
set fltap4 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]



set tap2sign ""
set tap3sign ""
set tap4sign ""
set tap5sign ""
set tap6sign ""
set tap7sign ""

set fltap1sign ""
set fltap2sign ""
set fltap3sign ""
set fltap4sign ""


if {[expr $tap2 >> 7] == 1} {
  set tap2sign -
}
if {[expr $tap3 >> 7] == 1} {
  set tap3sign -
}
if {[expr $tap4 >> 6] == 1} {
  set tap4sign -
}
if {[expr $tap5 >> 6] == 1} {
  set tap5sign -
}
if {[expr $tap6 >> 5] == 1} {
  set tap6sign -
}
if {[expr $tap7 >> 5] == 1} {
  set tap7sign -
}
if {[expr $fltap1 >> 6] == 1} {
  set fltap1sign -
}
if {[expr $fltap2 >> 6] == 1} {
  set fltap2sign -
}
if {[expr $fltap3 >> 5] == 1} {
  set fltap3sign -
}
if {[expr $fltap4 >> 5] == 1} {
  set fltap4sign -
}


set tap1mag [expr $tap1 & 0x7F]
set tap2mag [expr $tap2 & 0x7F]
set tap3mag [expr $tap3 & 0x7F]
set tap4mag [expr $tap4 & 0x3F]
set tap5mag [expr $tap5 & 0x3F]
set tap6mag [expr $tap6 & 0x1F]
set tap7mag [expr $tap7 & 0x1F]

set fltap1mag [expr $fltap1 & 0x3F]
set fltap2mag [expr $fltap2 & 0x3F]
set fltap3mag [expr $fltap3 & 0x3F]
set fltap4mag [expr $fltap4 & 0x3F]





				# Print info
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				set identifier [rd $m $base_addr 0x200]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				#puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $i	"
				#puts -nonewline "base_addr : [format 0x%x $base_addr]	"

puts -nonewline [format "%s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s |" "DFE taps:" "tap1=" 3 $tap1mag "tap2=" 4 $tap2sign$tap2mag "tap3="  4 $tap3sign$tap3mag "tap4=" 4 $tap4sign$tap4mag "tap5=" 4 $tap5sign$tap5mag "tap6=" 3 $tap6sign$tap6mag "tap7=" 3 $tap7sign$tap7mag "tap8=" 4 $fltap1sign$fltap1mag "tap9="  4 $fltap2sign$fltap2mag "tap10=" 4 $fltap3sign$fltap3mag "tap11=" 4 $fltap4sign$fltap4mag]

puts ""

}}}}
close_service slave $m
}
puts {show_dfe_taps_all {}}
puts {example useage show_dfe_taps_all}
puts ""

###############################################################################
#Show DFE taps of a specific channel in a PHY
###############################################################################

proc show_dfe_taps_channel {phy index channel} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""] 
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 

				set base_addr [expr 0x1000 * $channel]
		


#-----------------------DFE readback-----------------------#
#{0x71,0x01,0x1E,0x00}
#address 0x171 for top level testbus_sel - 0x3 <<1 for DFE
#rmw $m $base_addr 0x171 0x1e 0x06
#dfe outputs
rmw $m $base_addr 0x171 0x1e 0x14

#Address: 0x149[5:0] select bits 
#tap 1 = 011_1_1_0
#tap 2 = 011_1_1_1
#tap 3 = 100_0_0_0
#tap 4 = 100_0_0_1
#tap 5 = 100_0_1_0
#tap 6 = 100_0_1_1
#tap 7 = 100_1_0_0

#0x130 [3:0] sel_probe_tstmux
#probe_fxtap1_coeff	4'b0001
#probe_fxtap2_coeff	4'b0010
#probe_fxtap3_coeff	4'b0011
#probe_fxtap4_coeff	4'b0100
#probe_fxtap5_coeff	4'b0101
#probe_fxtap6_coeff	4'b0110
#probe_fxtap7_coeff	4'b0111
#probe_fltap1_coeff 4'b1011
#probe_fltap2_coeff 4'b1100
#probe_fltap3_coeff 4'b1101
#probe_fltap4_coeff 4'b1110


rmw $m $base_addr 0x130 0x0F 0x1
set tap1 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x130 0x0F 0x2
set tap2 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x130 0x0F 0x3
set tap3 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x130 0x0F 0x4
set tap4 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x130 0x0F 0x5
set tap5 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x130 0x0F 0x6
set tap6 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x130 0x0F 0x7
set tap7 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]


#floating tap readout is different?
#dfe outputs
rmw $m $base_addr 0x171 0x1e 0x16

rmw $m $base_addr 0x14C 0x3f 0x25
#rmw $m $base_addr 0x130 0x0F 0xB
set fltap1 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x14C 0x3f 0x26
#rmw $m $base_addr 0x130 0x0F 0xC
set fltap2 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x14C 0x3F 0x27
#rmw $m $base_addr 0x130 0x0F 0xD
set fltap3 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x14C 0x3F 0x28
#rmw $m $base_addr 0x130 0x0F 0xE
set fltap4 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]



set tap2sign ""
set tap3sign ""
set tap4sign ""
set tap5sign ""
set tap6sign ""
set tap7sign ""

set fltap1sign ""
set fltap2sign ""
set fltap3sign ""
set fltap4sign ""


if {[expr $tap2 >> 7] == 1} {
  set tap2sign -
}
if {[expr $tap3 >> 7] == 1} {
  set tap3sign -
}
if {[expr $tap4 >> 6] == 1} {
  set tap4sign -
}
if {[expr $tap5 >> 6] == 1} {
  set tap5sign -
}
if {[expr $tap6 >> 5] == 1} {
  set tap6sign -
}
if {[expr $tap7 >> 5] == 1} {
  set tap7sign -
}
if {[expr $fltap1 >> 6] == 1} {
  set fltap1sign -
}
if {[expr $fltap2 >> 6] == 1} {
  set fltap2sign -
}
if {[expr $fltap3 >> 5] == 1} {
  set fltap3sign -
}
if {[expr $fltap4 >> 5] == 1} {
  set fltap4sign -
}


set tap1mag [expr $tap1 & 0x7F]
set tap2mag [expr $tap2 & 0x7F]
set tap3mag [expr $tap3 & 0x7F]
set tap4mag [expr $tap4 & 0x3F]
set tap5mag [expr $tap5 & 0x3F]
set tap6mag [expr $tap6 & 0x1F]
set tap7mag [expr $tap7 & 0x1F]

set fltap1mag [expr $fltap1 & 0x3F]
set fltap2mag [expr $fltap2 & 0x3F]
set fltap3mag [expr $fltap3 & 0x3F]
set fltap4mag [expr $fltap4 & 0x3F]





				# Print info
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				set identifier [rd $m $base_addr 0x200]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				#puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $channel	"
				#puts -nonewline "base_addr : [format 0x%x $base_addr]	"

puts -nonewline [format "%s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s |" "DFE taps:" "tap1=" 3 $tap1mag "tap2=" 4 $tap2sign$tap2mag "tap3="  4 $tap3sign$tap3mag "tap4=" 4 $tap4sign$tap4mag "tap5=" 4 $tap5sign$tap5mag "tap6=" 3 $tap6sign$tap6mag "tap7=" 3 $tap7sign$tap7mag "tap8=" 4 $fltap1sign$fltap1mag "tap9="  4 $fltap2sign$fltap2mag "tap10=" 4 $fltap3sign$fltap3mag "tap11=" 4 $fltap4sign$fltap4mag]

puts ""

}}}}
close_service slave $m
}

puts {show_dfe_taps_channel {phy index channel}}
puts {example useage show_dfe_taps_channel $phy 1 2}
puts ""

###############################################################################
#Show DFE taps of a specific channel in a PHY for #loopcount times in a row
###############################################################################


proc show_dfe_taps_channel_loop {phy index channel loopcount} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 

				set base_addr [expr 0x1000 * $channel]
		
for {set i 0} {$i < $loopcount} {incr i} {


#-----------------------DFE readback-----------------------#
#{0x71,0x01,0x1E,0x00}
#address 0x171 for top level testbus_sel - 0x3 <<1 for DFE
#rmw $m $base_addr 0x171 0x1e 0x06
#dfe outputs
rmw $m $base_addr 0x171 0x1e 0x14

#Address: 0x149[5:0] select bits 
#tap 1 = 011_1_1_0
#tap 2 = 011_1_1_1
#tap 3 = 100_0_0_0
#tap 4 = 100_0_0_1
#tap 5 = 100_0_1_0
#tap 6 = 100_0_1_1
#tap 7 = 100_1_0_0

#0x130 [3:0] sel_probe_tstmux
#probe_fxtap1_coeff	4'b0001
#probe_fxtap2_coeff	4'b0010
#probe_fxtap3_coeff	4'b0011
#probe_fxtap4_coeff	4'b0100
#probe_fxtap5_coeff	4'b0101
#probe_fxtap6_coeff	4'b0110
#probe_fxtap7_coeff	4'b0111
#probe_fltap1_coeff 4'b1011
#probe_fltap2_coeff 4'b1100
#probe_fltap3_coeff 4'b1101
#probe_fltap4_coeff 4'b1110


rmw $m $base_addr 0x130 0x0F 0x1
set tap1 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x130 0x0F 0x2
set tap2 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x130 0x0F 0x3
set tap3 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x130 0x0F 0x4
set tap4 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x130 0x0F 0x5
set tap5 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x130 0x0F 0x6
set tap6 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x130 0x0F 0x7
set tap7 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]


#floating tap readout is different?
#dfe outputs
rmw $m $base_addr 0x171 0x1e 0x16

rmw $m $base_addr 0x14C 0x3f 0x25
#rmw $m $base_addr 0x130 0x0F 0xB
set fltap1 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x14C 0x3f 0x26
#rmw $m $base_addr 0x130 0x0F 0xC
set fltap2 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x14C 0x3F 0x27
#rmw $m $base_addr 0x130 0x0F 0xD
set fltap3 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]
rmw $m $base_addr 0x14C 0x3F 0x28
#rmw $m $base_addr 0x130 0x0F 0xE
set fltap4 [master_read_8 $m [expr 0x176*4 + $base_addr] 1]



set tap2sign ""
set tap3sign ""
set tap4sign ""
set tap5sign ""
set tap6sign ""
set tap7sign ""

set fltap1sign ""
set fltap2sign ""
set fltap3sign ""
set fltap4sign ""


if {[expr $tap2 >> 7] == 1} {
  set tap2sign -
}
if {[expr $tap3 >> 7] == 1} {
  set tap3sign -
}
if {[expr $tap4 >> 6] == 1} {
  set tap4sign -
}
if {[expr $tap5 >> 6] == 1} {
  set tap5sign -
}
if {[expr $tap6 >> 5] == 1} {
  set tap6sign -
}
if {[expr $tap7 >> 5] == 1} {
  set tap7sign -
}
if {[expr $fltap1 >> 6] == 1} {
  set fltap1sign -
}
if {[expr $fltap2 >> 6] == 1} {
  set fltap2sign -
}
if {[expr $fltap3 >> 5] == 1} {
  set fltap3sign -
}
if {[expr $fltap4 >> 5] == 1} {
  set fltap4sign -
}


set tap1mag [expr $tap1 & 0x7F]
set tap2mag [expr $tap2 & 0x7F]
set tap3mag [expr $tap3 & 0x7F]
set tap4mag [expr $tap4 & 0x3F]
set tap5mag [expr $tap5 & 0x3F]
set tap6mag [expr $tap6 & 0x1F]
set tap7mag [expr $tap7 & 0x1F]

set fltap1mag [expr $fltap1 & 0x3F]
set fltap2mag [expr $fltap2 & 0x3F]
set fltap3mag [expr $fltap3 & 0x3F]
set fltap4mag [expr $fltap4 & 0x3F]





				# Print info
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				set identifier [rd $m $base_addr 0x200]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				#puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $channel	"
				#puts -nonewline "base_addr : [format 0x%x $base_addr]	"

puts -nonewline [format "%s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s | %s %*s |" "DFE taps:" "tap1=" 3 $tap1mag "tap2=" 4 $tap2sign$tap2mag "tap3="  4 $tap3sign$tap3mag "tap4=" 4 $tap4sign$tap4mag "tap5=" 4 $tap5sign$tap5mag "tap6=" 3 $tap6sign$tap6mag "tap7=" 3 $tap7sign$tap7mag "tap8=" 4 $fltap1sign$fltap1mag "tap9="  4 $fltap2sign$fltap2mag "tap10=" 4 $fltap3sign$fltap3mag "tap11=" 4 $fltap4sign$fltap4mag]

puts ""

}}}}}
close_service slave $m
}

puts {show_dfe_taps_channel_loop {phy index channel loopcount}}
puts {example useage show_dfe_taps_channel_loop $phy 1 2 1000}
puts ""


###############################################################################
#Enable Floating taps to $channel in $phy
###############################################################################
proc enable_floating_taps {phy index channel} {

foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {

		# Match the phy
			if {[string last [lindex $phy $index ] $marker_info] != -1 } { 

set base_addr [expr [expr 0x1000 * $channel] ]

				
#-----------------------enable floating taps by disabling powerdown bit -----------------------#
#register 0x123[2]
rmw $m $base_addr 0x123 0x04 0x04
 


# Print info
set identifier [rd $m $base_addr 0x200]
set slave_addr [dict get $marker_info BASE_ADDRESS]
puts -nonewline "Slave: [format 0x%x $slave_addr]  "
puts -nonewline "Identifier: $identifier  "
puts -nonewline "channel  : $channel	"
puts -nonewline "base_addr : [format 0x%x [expr $base_addr ]]	"
puts -nonewline "	Floating Taps enabled  " 
puts ""
}}}}
close_service slave $m
}

puts {enable_floating_taps {phy index channel}}
puts {example useage enable_floating_taps $phy 0 1}
puts ""

###############################################################################
#Get Rx Offset from all channels
###############################################################################
proc get_offset_all {} {
foreach m [get_service_paths slave] {

	if {[string last slave $m] != -1 } {
      # Determine from text in the marker info if m is a Native PHY and only continue if it is
      # The marker will exist on the path of $m right now, but we are going to change the path in a second so do this now.
         set marker_info [marker_get_info $m] 
         set m [claim_service slave $m ""]
		if {[string last altera_xcvr_native_a10 $marker_info] != -1  } {
		
			set num_channels [rd $m 0x0 0x210]

			# Loop through all channels in this Native PHY
			for {set i 0} {$i < $num_channels } {incr i} {
				 set base_addr [expr 0x1000 * $i]

#Up values: 1 33 65 97 129 161 193 37 69 101 133 165 197 229 
#Center: 0
#Down values: 2 34 66 98 130 162 194 38 70 102 134 166 198 230


				# Print info
				set slave_addr [dict get $marker_info BASE_ADDRESS]
				set identifier [rd $m $base_addr 0x200]
				puts -nonewline "Slave: [format 0x%x $slave_addr]  "
				#puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $i	"
				#puts -nonewline "base_addr : [format 0x%x $base_addr]	"
				#puts -nonewline "	Offsets:" 
				set read_back [rd $m $base_addr 0x125]
				puts -nonewline "	0x125: " 	
				decode_offset [expr $read_back];
				#puts -nonewline  [format 0x%x $read_back ]
				set read_back [rd $m $base_addr 0x126]
				puts -nonewline "	0x126: " 
				decode_offset [expr $read_back];
				#puts -nonewline  [format 0x%x $read_back ]
				set read_back [rd $m $base_addr 0x127]
				puts -nonewline "	0x127: " 	
				decode_offset [expr $read_back];
				#puts -nonewline  [format 0x%x $read_back ]
				set read_back [rd $m $base_addr 0x128]
				puts -nonewline "	0x128: " 	
				decode_offset [expr $read_back];
				#puts -nonewline  [format 0x%x $read_back ]
				set read_back [rd $m $base_addr 0x129]
				puts -nonewline "	0x129: " 
				decode_offset [expr $read_back];
				#puts -nonewline  [format 0x%x $read_back ]
				set read_back [rd $m $base_addr 0x12a]
				puts -nonewline "	0x12A: " 
				decode_offset [expr $read_back];
				#puts -nonewline  [format 0x%x $read_back ]
				set read_back [rd $m $base_addr 0x12b]
				puts -nonewline "	0x12B: " 
				decode_offset [expr $read_back];
				#puts -nonewline  [format 0x%x $read_back ]
				set read_back [rd $m $base_addr 0x12c]
				puts -nonewline "	0x12C: " 
				decode_offset [expr $read_back];
				#puts -nonewline  [format 0x%x $read_back ]				
				puts ""
}}}}
close_service slave $m
}
puts {get_offset_all {}}
puts {example useage get_offset_all}
puts ""


#Up values: 1 33 65 97 129 161 193 37 69 101 133 165 197 229 
#Center: 0
#Down values: 2 34 66 98 130 162 194 38 70 102 134 166 198 230

proc decode_offset {offset} {
    switch $offset {
        0  { puts -nonewline " 0	" }
        1  { puts -nonewline " +1	" }		  
        33  { puts -nonewline " +2	" }
        65  { puts -nonewline " +3	" }
        97  { puts -nonewline " +4	" }
        129  { puts -nonewline " +5	" }
        161  { puts -nonewline " +6	" }
        193  { puts -nonewline " +7	" }
        37  { puts -nonewline " +8	" }
        69  { puts -nonewline " +9	" }
        101  { puts -nonewline " +a	" }
        133  { puts -nonewline " +b	" }
        165  { puts -nonewline " +c	" }
        197  { puts -nonewline " +d	" }
        229  { puts -nonewline " +e	" }
        2  { puts -nonewline " -1	" }
        34  { puts -nonewline " -2	" }	
        66  { puts -nonewline " -3	" }	
        98  { puts -nonewline " -4	" }	
        130  { puts -nonewline " -5	" }	
        162  { puts -nonewline " -6	" }	
        194  { puts -nonewline " -7	" }	
        38  { puts -nonewline " -8	" }	
        70  { puts -nonewline " -9	" }	
        102 { puts -nonewline " -a	" }	
        134  { puts -nonewline " -b	" }			  
        166  { puts -nonewline " -c	" }	
        198  { puts -nonewline " -d	" }	
        230  { puts -nonewline " -e	" }			  
		  default {puts -nonewline [format 0x%x $offset]; puts -nonewline "	"	}
    }
}



###############################################################################
# Start a PRBS-31 test on all channels and enable serial loopback
###############################################################################

proc start_check_serial_loop {} {

# enable serial loopback on all channels
set_serial_loop_all 1


# set prbs31 generator pattern
rmw_all 0x07 0xF0 0x00
rmw_all 0x08 0x10 0x10

# set prbs31 verifier pattern
rmw_all 0x0B 0xF0 0x00
rmw_all 0x0C 0x01 0x01


#SET GENERATOR
	#PMA data output is prbs pattern
rmw_all 0x06 0x0F 0x04
	#TX PCS PRBS Gen clock enable set to 1
rmw_all 0x06 0x40 0x40 
	#set serializer to 64-bit
rmw_all 0x110 0x07 0x03 

#Route PRBS to data path:
	# set rx_prbs_mode to 10g mode
rmw_all 0xB 0x02 0x02  
	#rx_prbs_force_signal_ok
rmw_all 0xC 0x02 0x02 
	#prbs9_dwidth set to prbs9_64b
rmw_all 0xC 0x08 0x00
	# Set deser_factor to 64-bit
rmw_all 0x13F 0x0F 0x0E

# TX PCS PRBS Gen clock enable set to on
rmw_all 0xA 0x80 0x80

# Counter enable (enables both error and bit counters)
rmw_all 0x300 0x01 0x01

}
puts {start_check_serial_loop {}}
puts {example useage start_check_serial_loop }
puts ""

###############################################################################
# Start a PRBS-31 test on all channels 
###############################################################################


proc start_check_prbs31 {} {

# set prbs31 generator pattern
rmw_all 0x07 0xF0 0x00
rmw_all 0x08 0x10 0x10

# set prbs31 verifier pattern
rmw_all 0x0B 0xF0 0x00
rmw_all 0x0C 0x01 0x01


#SET GENERATOR
	#PMA data output is prbs pattern
rmw_all 0x06 0x0F 0x04
	#TX PCS PRBS Gen clock enable set to 1
rmw_all 0x06 0x40 0x40 
	#set serializer to 64-bit
rmw_all 0x110 0x07 0x03 

#Route PRBS to data path:
	# set rx_prbs_mode to 10g mode
rmw_all 0xB 0x02 0x02  
	#rx_prbs_force_signal_ok
rmw_all 0xC 0x02 0x02 
	#prbs9_dwidth set to prbs9_64b
rmw_all 0xC 0x08 0x00
	# Set deser_factor to 64-bit
rmw_all 0x13F 0x0F 0x0E

# TX PCS PRBS Gen clock enable set to on
rmw_all 0xA 0x80 0x80

# Counter enable (enables both error and bit counters)
rmw_all 0x300 0x01 0x01

}

puts {start_check_prbs31 {}}
puts {example useage start_check_prbs31 }
puts ""

###############################################################################
# The following are examples and can be used as a reference.
###############################################################################

proc start_test {} {
#set_cdr_gear_all 31 3 3
set_ac_gain_all 3
# temp workaround to disable Tx flatlining
#rmw_all 0x106 0x60 0x60
#rmw_all 0x107 0x60 0x60
#rmw_all 0x115 0x57 0x57
# set cgm_bias_disable 0x11B bit [5] to 1 to improve Rx sensitivity
rmw_all 0x11B 0x20 0x20
# ES1 set chgpmp_replicate 0x139 bit [6] to  0
rmw_all 0x139 0x40 0x0
# ES2 set chgpmp_replicate 0x139 bit [6] to 1
#rmw_all 0x139 0x40 0x40
start_check_prbs31
}



###############################################################################
# Displays the PHY's found in the design
###############################################################################

puts {}
puts {These are the Native PHY's identified in the design : }
puts {You can use the index of the phy to identify a certain phy }
set phy [show_phys]

#for {set i 0} {$i < [llength $phy]} {incr i}  {
# puts [lindex $phy  $i]
#}

